/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.sprites;

import java.awt.*;
import java.awt.image.*;

/**
 * A clipped image sprite.  The sprite consists of an image and the segment
 * of the image that is to be drawn.
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: June 9, 1998
 */
public class ImageSprite implements SpriteProducer {
// *** VARIABLES ***
  Image m_iImage;
  int m_nX, m_nY, m_nW, m_nH;

  public ImageSprite() {
  }

  public ImageSprite(Image i, int nx, int ny, int nw, int nh) {
    m_iImage = i;
    m_nX = nx;
    m_nY = ny;
    m_nW = nw;
    m_nH = nh;
  }

  // Sets an image (with it's clipped area) for the sprite
  public void setImage(Image i, int nx, int ny, int nw, int nh) {
    m_iImage = i;
    m_nX = nx;
    m_nY = ny;
    m_nW = nw;
    m_nH = nh;
  }

  // Changes the clipped area of the sprite
  public void setClip(int nx, int ny, int nw, int nh) {
    m_nX = nx;
    m_nY = ny;
    m_nW = nw;
    m_nH = nh;
  }

	// Handles drawing contents at provided coordinates
	public void relativePaint(Graphics g, int nX, int nY, ImageObserver observer){
    g.drawImage(m_iImage, nX, nY, nX + m_nW, nY + m_nH,
          m_nX, m_nY, m_nX + m_nW, m_nY + m_nH, observer);
  }

	// Returns the width of the component
	public int getWidth() {
    return m_nW;
  }

	// Returns the height of the component
	public int getHeight() {
    return m_nH;
  }
}