/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.sprites;
/*
	AUTHOR: Billy D. Spelchan
	CREATED: March 5, 1998
	DESCRIPTION:
	Interface for enabling sprite drawing.  All a class has to do to belong
	to this interface is be able to draw themselves at specified coodinates!
	It is the core interface for the sprite library and is also used by Zones
	to enable the drawing of dragging objects.
*/

import java.awt.*;
import java.awt.image.*;

/**
 * A clipped image sprite.  The sprite consists of an image and the segment
 * of the image that is to be drawn.
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: June 9, 1998
 */
public interface SpriteProducer {
	// Handles drawing contents at provided coordinates
	public abstract void relativePaint(Graphics g, int nX, int nY, ImageObserver observer);

	// Returns the width of the component
	public abstract int getWidth();

	// Returns the height of the component
	public abstract int getHeight();
}