/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.textfx;

import java.awt.*;

/**
 * Base class for text effects.  This class draws outlined text!
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: March 12, 1998
 */
public class TextFX {
// *** VARIABLES ***
   Font m_font;
           // the font to draw the text with
   Color m_colorText, m_colorOutline;
           // the colors to use when drawing the text

// *** CONSTRUCTORS ***
   // default constructor.  Uses 12 point font with red text outlined in black.
   public TextFX() {
       m_font = new Font("TimesRoman", Font.PLAIN, 12);
       m_colorText = Color.red;
       m_colorOutline = Color.black;
   }

   // Constructor using provided font, text color and outline color.
   public TextFX(Font f, Color cText, Color cOutline) {
       m_font = f;
       m_colorText = cText;
       m_colorOutline = cOutline;
   }

// *** MODIFICATION FUNCTIONS ***
   // sets the text color
   public void setTextColor(Color cText){
       m_colorText = cText;
   }

   // sets the outline color
   public void setOutlineColor(Color cOutline){
       m_colorOutline = cOutline;
   }

   // sets the font to use when drawing the text
   public void setFont(Font f){
       m_font = f;
   }

// *** INFORMATION FUNCTIONS ***
   // returns the width a given string would be
   public int getWidth(Graphics g, String s) {
       FontMetrics fm = g.getFontMetrics(m_font);
       return fm.stringWidth(s) + 2;
   }

   // returns the standard height of the font
   public int getHeight(Graphics g) {
       FontMetrics fm = g.getFontMetrics(m_font);
       return fm.getHeight();
   }

// *** DRAWING FUNCTIONS ***
   // The routine that does all the drawing.
   public void drawLeft(Graphics g, int x, int y, String s) {
       int cntrX, cntrY;

       // Prepare for drawing
       Font fontOld = g.getFont();
       Color colorOld = g.getColor();

       // Do the drawing
       g.setFont(m_font);
       g.setColor(m_colorOutline);
       for (cntrX = 0; cntrX < 3; ++cntrX)
           for (cntrY = 0; cntrY < 3; ++cntrY)
               g.drawString(s, x+cntrX, y+cntrY);
       g.setColor(m_colorText);
       g.drawString(s, x+1, y+1);

       // clean up
       g.setFont(fontOld);
       g.setColor(colorOld);
   }

   // calls drawleft adjusting coordinates for right justification
   public void drawRight(Graphics g, int x, int y, String s) {
       drawLeft(g, x-getWidth(g, s), y, s);
   }

   // calls drawleft adjusting coordinates for centering text
   public void drawCentered(Graphics g, int x, int y, String s) {
       drawLeft(g, x-getWidth(g, s)/2, y, s);
   }
}
