/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.zones;

import java.awt.*;
//import java.awt.event.*;
import java.util.*;

/**
 * An extended version of the ZoneAdapter which handles the drawing of zones
 * and draggable objects.  IT SHOULD BE NOTED THAT IN ORDER TO HANLE DRAGGING,
 * ALL OF THE VISIBLE SCREEN MUST BE ZONES!
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: March 5, 1998
 */
public class DrawingZoneAdapter extends ZoneAdapter{
// *** CONSTANTS

// *** VARIABLES

// *** CONSTRUCTOR ***
	public DrawingZoneAdapter(ZoneListener zl) {
		super(zl);
	}

// *** SETUP & MODIFICATION METHODS ***
	// Goes through all the zones painting them.  Handles dragging by filling
	// Non-Zone areas with provided color while refreshing affected zones.
	public void paintZones(Graphics g, Container c, Color colorBackground) {
		Zone z;
		int nEvent;
		Rectangle rect;

		// First Pass - Fill dirty rectangles with background color
		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			if (z.checkDragDrawState()){
				rect = z.getDragDropDirtyRect();
				invalidateRect(rect.x, rect.y, rect.x+rect.width, rect.y+rect.height);
				g.setColor(colorBackground);
				g.fillRect(rect.x, rect.y, rect.x+rect.width, rect.y+rect.height);
			}
		}

		// Second Pass - Paint the zones
		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			nEvent = z.paint(g, c);
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);
		}

		// Third Pass - Paint the dragged items (or whatever)
		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			if (z.checkDragDrawState()){
				z.paintDrag(g, c);
				z.clearDragDrawState();
			}
		}
	}

}
