/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.zones;

import java.awt.*;
import java.awt.image.*;

/**
 * Uses a passed image to draw an image based on the buttons state.
 * There should be four images stored horizontally.
 * First represents normal state, second highlight state, third selected
 * and the fourth is disabled.
 * By using image offset feature, more than 1 button can be stored in a image.
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: June 19, 1998
 */
public class ImageButton extends Zone {
// *** CONSTANTS ***

// *** VARIABLES ***
	Image m_iButton;
  int m_nImageOffsetX, m_nImageOffsetY;

// *** CONSTRUCTOR ***
	public ImageButton(int nID, int nX, int nY, int nW, int nH, Image i) {
		super(nID, nX, nY, nW, nH, Zone.MASK_HIGHLIGHT | Zone.MASK_SELECTED | Zone.MASK_RELEASED );
    m_iButton = i;
    m_nImageOffsetX = 0;
    m_nImageOffsetY = 0;
	}

// *** MODIFICATION ***
  public void setImage(Image i) {
    m_iButton = i;
  }

  // Offset on the provided image for the buttons.  This allows one image file
  // to contain multiple buttons
  public void setImageOffset(int x, int y) {
    m_nImageOffsetX = x;
    m_nImageOffsetY = y;
  }

// *** INFORMATION ***
  // Get the button image
  public Image getImage() {
    return m_iButton;
  }

  // Get the current x offset into the button image
  public int getImageOffsetX() {
    return m_nImageOffsetX;
  }

  // Get the current y offset into the button image
  public int getImageOffsetY() {
    return m_nImageOffsetY;
  }

// *** DRAWING ***
	// Fully draws the component
	public int paint(Graphics g, ImageObserver observer) {
    if (!isVisible())
      return 0;

    int nOffX;
    int nW = getWidth();
    int nH = getHeight();

		super.paint(g, observer);

    switch (m_nState) {
      case STATE_DISABLED:
        nOffX = 3 * nW + m_nImageOffsetX;
        break;
      case STATE_ACTIVE:
        nOffX = m_nImageOffsetX;
        break;
      case STATE_HIGHLIGHTED:
        nOffX = m_nImageOffsetX + nW;
        break;
      case STATE_SELECTED:
        nOffX = m_nImageOffsetX + nW * 2;
        break;
      default :
        nOffX = 0;
    }
    g.drawImage(m_iButton, m_nX, m_nY, m_nX2, m_nY2,
          nOffX, m_nImageOffsetY, nOffX + nW, m_nImageOffsetY + nH, observer);
    return 0;
	}
}