/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.zones;

import java.awt.*;
import java.awt.image.*;

import com.blazinggames.common.sprites.*;

/**
 * Zones text button
 *  
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: January 12, 1999
 */
public class TextButton extends Zone {
// *** CONSTANTS ***

// *** VARIABLES ***
  TextSprite tsText;
  Color colorShadow;
  Color colorNormal, colorHighlight, colorSelected;
  Color colorBoxNormal, colorBoxHighlight, colorBoxSelected;

  int nBoxBorderSize;
  boolean bButtonBoxVisible;

// *** CONSTRUCTOR ***
	public TextButton(int nID, int nX, int nY, int nW, int nH, String text) {
		super(nID, nX, nY, nW, nH, Zone.MASK_HIGHLIGHT | Zone.MASK_SELECTED | Zone.MASK_RELEASED );

    tsText = new TextSprite(TextSprite.DEFAULTFONT, text, Color.black, nW, nH);
    tsText.setShadow(Color.blue, Color.black, 1);
    tsText.setText(text, TextSprite.ALIGNCENTER);

    colorShadow = Color.black;
    colorNormal = Color.blue;
    colorHighlight = Color.cyan;
    colorSelected = Color.red;

    colorBoxNormal = Color.gray;
    colorBoxHighlight = Color.gray;
    colorBoxSelected = Color.darkGray;

    nBoxBorderSize = 3;
    bButtonBoxVisible = true;
	}

// *** MODIFICATION ***
  public void setText(String text) {
    tsText.setText(text, TextSprite.ALIGNCENTER);
  }

  public void setFont(Font f) {
    tsText.setFont(f);
  }

  public void setTextColors(Color norm, Color high, Color sel, Color shadow) {
    colorNormal = norm;
    colorHighlight = high;
    colorSelected = sel;
    colorShadow = shadow;
  }

  public void setBoxColors(Color norm, Color high, Color sel) {
    colorBoxNormal = norm;
    colorBoxHighlight = high;
    colorBoxSelected = sel;
  }

  public void setBoxBorders(int size) {
    nBoxBorderSize = size;
  }

  public void showButtonBox(boolean bbstate) {
    bButtonBoxVisible = bbstate;
  }

// *** INFORMATION ***

// *** DRAWING ***
	// Fully draws the component
	public int paint(Graphics g, ImageObserver observer) {
		super.paint(g, observer);
    if (!isVisible())
      return 0;

    boolean b3DMode = true;

    switch (m_nState) {
      case STATE_DISABLED:
        g.setColor(Color.darkGray);
        tsText.setTextColor(Color.gray);
        break;
      case STATE_ACTIVE:
        g.setColor(colorBoxNormal);
        tsText.setTextColor(colorNormal);
        break;
      case STATE_HIGHLIGHTED:
        g.setColor(colorBoxHighlight);
        tsText.setTextColor(colorHighlight);
        break;
      case STATE_SELECTED:
        g.setColor(colorBoxSelected);
        tsText.setTextColor(colorSelected);
        b3DMode = false;
        break;
      default :
    }

    if (bButtonBoxVisible) {
      for (int cntr = 0; cntr < nBoxBorderSize; ++cntr)
        g.draw3DRect(m_nX + cntr, m_nY + cntr, getWidth() - cntr*2,
              getHeight() - cntr*2, b3DMode);
      g.fillRect(m_nX + nBoxBorderSize, m_nY + nBoxBorderSize,
            getWidth() - nBoxBorderSize*2+1, getHeight()- nBoxBorderSize*2+1);
    }
    tsText.relativePaint(g, m_nX, m_nY, observer);
    return 0;
	}
}