package com.blazinggames.fleet;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.Hashtable;

/**
 * Fleet
 * Copyright (C) 1998, 2002, 2006 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * The Fleet game applet 
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class Fleet
extends Applet
implements Runnable, MouseListener, MouseMotionListener, KeyListener
{
// *********************
// ***** CONSTANTS *****
// *********************
    
    private static final Font ERROR_FONT = new Font("Serif", Font.PLAIN, 32);
    private static final String ERROR_TEXT = "AN ERROR OCCURRED:";
    private static final Color ERROR_COLOR = Color.yellow;
    private static final int ERROR_MAINBASELINE = 200;
    private static final int ERROR_DESCBASELINE = 300;

    // Parameters
    private static final String TITLE = "TITLE";
    private static final String VSHIP = "VSHIP";
    private static final String HSHIP = "HSHIP";
    private static final String SPLASH = "SPLASH";
    private static final String HIT = "HIT";
    private static final String WIN = "WIN";
    private static final String LOSE = "LOSE";
    private static final String PLAYGRIDX = "PLAYGRIDX";
    private static final String PLAYGRIDY = "PLAYGRIDY";
    private static final String PLAYWIDTH = "PLAYWIDTH";
    private static final String PLAYHEIGHT = "PLAYHEIGHT";
    private static final String OPPGRIDX = "OPPGRIDX";
    private static final String OPPGRIDY = "OPPGRIDY";
    private static final String OPPWIDTH = "OPPWIDTH";
    private static final String OPPHEIGHT = "OPPHEIGHT";
    private static final String MESSAGEX = "MESSAGEX";
    private static final String MESSAGEY = "MESSAGEY";
    private static final String MESSAGESIZE = "MESSAGESIZE";
    private static final String SHIPSX = "SHIPSX";
    private static final String SHIPSY = "SHIPSY";
    private static final String SHIPSSIZE = "SHIPSSIZE";
    private static final String SOUND_SILENCE = "SNDSILENT";
    private static final String SOUND_SPLASH = "SNDSPLASH";
    private static final String SOUND_HIT = "SNDHIT";
    private static final String SOUND_SINK = "SNDSINK";
    private static final String SOUND_WIN = "SNDWIN";
    private static final String SOUND_LOSE = "SNDLOSE";
    private static final String SOUND_TOGGLE = "SNDTOGGLE";
    private static final String SOUND_TOGGLEX = "SNDTOGGLEX";
    private static final String SOUND_TOGGLEY = "SNDTOGGLEY";
    private static final String SOUND_TOGGLESIZE = "SNDTOGGLESIZE";
        
    // New Colors used in the game
    static final Color clNavy = new Color(0, 0, 128);
	static final Color clAqua = new Color(0, 128, 128);
	static final Color clLtBlue = new Color(128,128,255);

    // The potential game states
    static final int MODE_TITLE = 0;
	static final int MODE_PLACESHIPS = 1;
	static final int MODE_ANIMATEDTRANSFER = 2;
	static final int MODE_PLAYGAME = 3;
	static final int MODE_GAMEOVER = 4;
	static final int MODE_TITLETRANSFER = 5;

    // Win/Lose States
    static final int PLAYERWINS = 0;
    static final int COMPUTERWINS = 1;
    

// *********************
// ***** VARIABLES *****
// *********************
    
    // Layout
    private int _playGridX = 10;
    private int _playGridY = 20;
    private int _playWidth = 32;
    private int _playHeight = 32;
    private int _oppGridX = 338;
    private int _oppGridY = 20;
    private int _oppWidth = 16;
    private int _oppHeight = 16;
    private int _messageX = 10;
    private int _messageY = 14;
    private Font _messageFont = null;
    private int _shipsX = 410;
    private int _shipsY = 250;
    private int _shipsSize = 16;
    private Font _shipsFont = null;
    
    Grid m_gridPlayer;
	Opponent m_gridOpponent;
			// Game objects
	Image iShipV[] = new Image[5];
	Image iShipH[] = new Image[5];
	Image iSplash, iHit, iTitle, iComputerWins, iPlayerWins;
			// Graphics
	int mouseX, mouseY;
	int m_nShipsPlaced;//, m_nGameMode;
	int m_nCurX, m_nCurY, m_nCurD;
	int m_nOldX, m_nOldY, m_nOldD;
  String m_sMessage;
			// Interface objects
	int m_nMode, m_nWinState;
			// Current mode
	int m_nAnimX, m_nAnimY, m_nAnimW, m_nAnimH, m_nAnimPhase;
//	int m_nCurFrame, m_nLastFrame;
	boolean m_bAnimationPlaying, m_bFullDrawNeeded;
			// Animation variables
	Thread runner;

    private MediaTracker _tracker;
    private int _loadedPercent;

    private String errorMessage = "unknown";
    private boolean errorOccured = false;
    private Hashtable _config = new Hashtable();

    // sound
    private boolean _playSoundToggle;
    private AudioClip _soundSilence;
    private AudioClip _soundSplash;
    private AudioClip _soundHit;
    private AudioClip _soundSink;
    private AudioClip _soundWin;
    private AudioClip _soundLose;
    private Image _soundImage;
    private Rectangle _soundToggle;
    
// *** THREAD HANDLING ***
	//Initialize the applet
	public void init()
    {
		int cntr;
		String sName;

        _loadedPercent = 0;
        
//        loadConfig();
        new Thread(new FleetLoader(this)).start();
		// Prepare the grids
		m_gridPlayer = new Grid();
		m_gridPlayer.randomShips();
		m_gridOpponent = new Opponent();
		m_nShipsPlaced = m_nMode = 0;
		m_nCurX = m_nCurY = m_nOldX = m_nOldY = -1;
		m_nCurD = m_nOldD = 0;
		m_bAnimationPlaying = false;
		m_bFullDrawNeeded = true;
		this.setBackground(clLtBlue);
        addMouseListener(this);
        addMouseMotionListener(this);
        addKeyListener(this);
        requestFocus();
	}

  private void startGame() {
		m_gridPlayer.initGrid();
		m_gridPlayer.randomShips();
		m_gridOpponent.initGrid();
		m_nShipsPlaced = 0;
		m_nCurX = m_nCurY = m_nOldX = m_nOldY = -1;
		m_nCurD = m_nOldD = 0;
  }
	// Start running the main
	public void start() {
		if (runner == null) {
			runner = new Thread(this);
			runner.start();
		}
	}

	// animation thread
	public void run() {
		repaint();
		while (true) {
			if (m_bAnimationPlaying) {
//				++nFrameCount;
				repaint();
			}
			try { Thread.sleep(50); }
			catch (InterruptedException e) { }
		}
	}

	public void destroy()
    {
		if (runner != null)
        {
			runner = null;
		}
        super.destroy();
	}

    public String getAppletInfo()
    {
    	return "Fleet V1.00 OSR. Copyright 1998,2002,2006 Blazing Games Inc.";
	}


// *** DRAWING RELATED ***
	public void update(Graphics g)
    {
        Dimension d = getSize();
		if ((m_bFullDrawNeeded) || (errorOccured))
        {
			g.setColor(getBackground());
			g.fillRect(0,0, d.width,d.height);
			g.setColor(getForeground());
		}
        paint(g);
	}

//---------------------------------------------------------------------------
	private void paint_Title(Graphics g)
    {
		// Title
        if (iTitle != null)
        {
            Dimension d = getSize();
            int w = iTitle.getWidth(this);
            int h = iTitle.getHeight(this);
            g.drawImage(iTitle, (d.width - w) / 2, (d.height - h) / 2, this);
        }

        // load bar
        if (_loadedPercent < 100)
        {
            g.setColor(Color.black);
            g.drawString("Please wait...loading", 120,300);
            g.drawRect(120,320,400,30);
            g.fillRect(120,320,4*_loadedPercent,30);
        }
	}

	private void paint_Placement(Graphics g)
    {
    int nW, nH;
    int X = (mouseX - _playGridX) / _playWidth;
    int Y = (mouseY - _playGridY) / _playHeight;
    int D = m_nCurD;

    if ( (X != m_nOldX) || (Y != m_nOldY) || (D != m_nOldD) )
    {
      m_bFullDrawNeeded = false;
      drawGrid(g,0,0,9,9);

      // Draw new placement
      if ((X >= 0) && (X < 10) && (Y >= 0) && (Y < 10) ) {
        if (m_gridOpponent.checkPlacement(m_nShipsPlaced, X, Y, D))
          g.setColor(Color.white);
        else
          g.setColor(Color.red);
        if (D == 1) {
          nW = Math.min(Grid.cm_nShipSize[m_nShipsPlaced], 10-X);
          nW = nW * _playWidth - 1;
          g.fillRect(X * _playWidth + _playGridX,Y * _playHeight + _playGridY,
                nW, _playWidth-1);
        } else {
          nH = Math.min(Grid.cm_nShipSize[m_nShipsPlaced], 10-Y);
          nH = nH * _playHeight - 1;
          g.fillRect(X * _playWidth + _playGridX,Y * _playHeight + _playGridY,
                _playHeight, nH);
        }
      }
      // update position variables
      m_nOldX = m_nCurX;
      m_nCurX = X;
      m_nOldY = m_nCurY;
      m_nCurY = Y;
      m_nOldD = m_nCurD;
      m_nCurD = D;
      m_sMessage = "Place your " + Grid.c_szShipNames[m_nShipsPlaced];
      drawMessage(g);
    }
	}

    private void paint_ShipKey(Graphics g)
    {
        String s;
        int curY = _shipsY;

        if (_shipsFont == null)
            return;
        g.setFont(_shipsFont);
        g.setColor(Color.yellow);
        for (int cntr = 0; cntr < 5; ++cntr)
        {
            g.drawString(Grid.c_szShipNames[cntr], _shipsX, curY);
            g.drawImage(iShipH[cntr], _shipsX, curY, this);
            curY += iShipH[cntr].getHeight(this);
            curY += _shipsSize;
        }
    }
    
	private void paint_AnimatedTransfer(Graphics g) {
		int cntr, temp;

		if (m_bFullDrawNeeded) {
			g.setColor(clNavy);
			g.fillRect(m_nAnimX, m_nAnimY, m_nAnimW, m_nAnimH);
			m_bFullDrawNeeded = false;
		} else if (m_nAnimPhase == 1)
			if (m_nAnimW >= (_playWidth * 10))
				nextMode();
			else {
				g.setColor(clNavy);
				for (cntr = 0; cntr < 4; ++cntr) {
					temp = (_playWidth*10 - m_nAnimW - cntr * 2) / 2;
					g.drawRect(_playGridX+temp, _playGridY+temp,
								m_nAnimW+cntr*2, m_nAnimW+cntr*2);
				}
				m_nAnimW += 8;
			}
		else {
			// Add chunk for movement
			g.setColor(clNavy);
			for (cntr = 0; cntr < 8; ++cntr)
				g.drawLine(m_nAnimX+m_nAnimW+cntr, m_nAnimY,
							m_nAnimX+m_nAnimW+cntr, m_nAnimY + m_nAnimH);
			// remove chunk for movement and chunk for reduction
			g.setColor(clLtBlue);
			for (cntr = 0; cntr < 8; ++cntr)
				g.drawLine(m_nAnimX+cntr, m_nAnimY,
							m_nAnimX+cntr, m_nAnimY + m_nAnimH);
			if (m_nAnimW > (_oppWidth * 10)) {
				for (cntr = 0; cntr < 4; ++cntr)
					g.drawLine(m_nAnimX, m_nAnimY + m_nAnimH-cntr,
								m_nAnimX+m_nAnimW+8, m_nAnimY + m_nAnimH-cntr);
				m_nAnimW -= 4;
				m_nAnimH -= 4;
			}
			m_nAnimX +=8;
			if (m_nAnimX >= _oppGridX) {
				m_nAnimPhase = 1;
				m_nAnimW = 0;
				m_nAnimH = 0;
			}

		}
	}

    private void paintError(Graphics g)
    {
        Dimension d = getSize();
        g.setFont(ERROR_FONT);
        g.setColor(ERROR_COLOR);
        FontMetrics fm = g.getFontMetrics();
        int x = (d.width - fm.stringWidth(ERROR_TEXT)) / 2;
        g.drawString(ERROR_TEXT, x, ERROR_MAINBASELINE);
        x = (d.width - fm.stringWidth(errorMessage)) / 2;
        g.drawString(errorMessage, x, ERROR_DESCBASELINE);
        
    }

	public void paint(Graphics g)
    {
        if (errorOccured)
        {
            paintError(g);
            return;
        }

        if ( (_soundImage != null) && (_soundToggle != null) )
        {
            int dx2 = _soundToggle.x + _soundToggle.width;
            int dy2 = _soundToggle.y + _soundToggle.height;
            int sx1 = _soundToggle.width;
            int sy1 = 0;
            int sx2 = sx1 + _soundToggle.width;
            int sy2 = _soundToggle.height;
            if (_playSoundToggle)
            {
                sx1 += _soundToggle.width;
                sx2 += _soundToggle.width;
                g.setColor(Color.red);
            }
            else
            {
                g.setColor(Color.black);
            }
            g.drawImage(_soundImage,
                        _soundToggle.x, _soundToggle.y, dx2, dy2,
                        sx1, sy1, sx2, sy2, this);
            g.drawRect(_soundToggle.x, _soundToggle.y, _soundToggle.width, _soundToggle.height);
        }
        switch (m_nMode) {
            case MODE_TITLE :
                paint_Title(g);
                break;
            case MODE_PLACESHIPS :
                paint_Placement(g);
                break;
            case MODE_ANIMATEDTRANSFER :
                paint_AnimatedTransfer(g);
                break;
            case MODE_PLAYGAME :
                drawMessage(g);
                paint_ShipKey(g);
                drawGrid(g,0,0,9,9);
                PBOpponentPaint(g);
                break;
            case MODE_GAMEOVER :
                paint_GameOver(g);
                break;
            case MODE_TITLETRANSFER :
                paint_TitleTransfer(g);
                break;
            default :
                paint_Title(g);
        }
	}

        
	private void paint_GameOver(Graphics g)
    {
        int w,h;
        Dimension d = getSize();
        
        drawMessage(g);
        drawGrid(g,0,0,9,9);
        PBOpponentPaint(g);
        if (m_nWinState == PLAYERWINS)
        {
            w = iPlayerWins.getWidth(this);
            h = iPlayerWins.getHeight(this);
            g.drawImage(iPlayerWins, (d.width - w) / 2, (d.height - h) / 2, this);
        }
        else
        {
            w = iComputerWins.getWidth(this);
            h = iComputerWins.getHeight(this);
            g.drawImage(iComputerWins, (d.width - w) / 2, (d.height - h) / 2, this);
        }
    }

	private void paint_TitleTransfer(Graphics g)
    {
		if ( (m_nAnimW <= 12) || (m_nAnimH <= 12) )
			nextMode();
		else
        {
			int cntr;
			g.setColor(Color.black);
			for (cntr = 4; cntr < 12; ++cntr)
				g.drawRect(m_nAnimX+cntr, m_nAnimY+cntr,
							m_nAnimW - cntr*2, m_nAnimH - cntr*2);
			g.setColor(clLtBlue);
			for (cntr = 0; cntr < 4; ++cntr)
				g.drawRect(m_nAnimX+cntr, m_nAnimY+cntr,
							m_nAnimW - cntr*2, m_nAnimH - cntr*2);
			m_nAnimW -= 8;
			m_nAnimH -= 8;
			m_nAnimX += 4;
			m_nAnimY += 4;
		}
	}

	/*
		draws a portion of the grid.  Coordinates are grid coordinates.
	*/
	void drawGrid(Graphics g, int x1, int y1, int x2, int y2)
	{
		int cntr, cntr2;

        if ((x1 < 0) || (x1 > 9) || (y1 < 0) || (y1 > 9) )
        return;
        x2 = Math.min(x2, 9);
        y2 = Math.min(y2, 9);
    
		// Draw up the players grid
		g.setColor(clNavy);
		g.fillRect(_playGridX + _playWidth*x1, _playGridY + _playHeight*y1, (x2-x1+1)*_playWidth-1,(y2-y1+1)*_playHeight-1);
		g.setColor(clAqua);
		for (cntr = x1; cntr <= x2; ++cntr)
			g.drawLine(_playGridX + _playWidth*cntr,_playGridY + _playHeight*y1,
						_playGridX + _playWidth*cntr,_playGridY + _playHeight*(y2+1));
		for (cntr = y1; cntr <= y2; ++cntr)
			g.drawLine(_playGridX + _playWidth*x1,_playGridY + _playHeight*cntr,
						_playGridX + _playWidth*(x2+1),_playGridY + _playHeight*cntr);

    if (m_nMode == MODE_PLACESHIPS) {
       // Draw the ships
       for (cntr = 0; cntr < m_nShipsPlaced; ++cntr)
           if (m_gridOpponent.isShipInRegion(cntr, x1, y1, x2, y2)) {
              if (m_gridOpponent.getShipD(cntr) == 0) // vertical
                 g.drawImage(iShipV[cntr],
                        _playGridX+_playWidth*m_gridOpponent.getShipX(cntr),
                        _playGridY+_playHeight*m_gridOpponent.getShipY(cntr),
                        _playWidth, Grid.cm_nShipSize[cntr] * _playHeight, this);
              else
                 g.drawImage(iShipH[cntr],
                        _playGridX+_playWidth*m_gridOpponent.getShipX(cntr),
                        _playGridY+_playHeight*m_gridOpponent.getShipY(cntr),
                        Grid.cm_nShipSize[cntr] * _playWidth, _playHeight, this);
           }
    } else {
       // show any shots
       for (cntr = 0; cntr < 10; ++cntr)
         for (cntr2 = 0; cntr2 < 10; ++cntr2)
           if (m_gridPlayer.getGrid(cntr, cntr2) > 0)
             if (m_gridPlayer.getGrid(cntr, cntr2) == 1)
               g.drawImage(iSplash,_playGridX+cntr*_playWidth,_playGridY+cntr2*_playHeight,this);
             else
               g.drawImage(iHit,_playGridX+cntr*_playWidth,_playGridY+cntr2*_playHeight,this);

       // Draw the ships
       for (cntr = 0; cntr < 5; ++cntr)
           if (m_gridPlayer.getSunkState(cntr) != 0) {
              if (m_gridPlayer.getShipD(cntr) == 0) // vertical
                 g.drawImage(iShipV[cntr],
                      _playGridX+_playWidth*m_gridPlayer.getShipX(cntr),
                      _playGridY+_playHeight*m_gridPlayer.getShipY(cntr),
                        _playWidth, Grid.cm_nShipSize[cntr] * _playHeight, this);
              else
                 g.drawImage(iShipH[cntr],
                      _playGridX+_playWidth*m_gridPlayer.getShipX(cntr),
                      _playGridY+_playHeight*m_gridPlayer.getShipY(cntr),
                        Grid.cm_nShipSize[cntr] * _playWidth, _playHeight, this);
           }
    }
	}

	private void drawMessage(Graphics g)
    {
        int cntr, cntr2;
        Dimension d = getSize();
      
        if (m_sMessage == null)
            return;
        g.setColor(clLtBlue);
        g.fillRect(0, 0, d.width, _playGridY);
        if (_messageFont != null)
            g.setFont(_messageFont);
        g.setColor(Color.black);
        for (cntr = 0; cntr < 3; ++cntr)
            for (cntr2 = 0; cntr2 < 3; ++cntr2)
                g.drawString(m_sMessage, _messageX + cntr, _messageY + cntr2);
        g.setColor(Color.red);
        g.drawString(m_sMessage, _messageX + 1, _messageY + 1);
    }
    
	void PBOpponentPaint(Graphics g)
	{
		int cntr, cntr2;

		// Draw up the Opponent grid
		g.setColor(clNavy);
		g.fillRect(_oppGridX, _oppGridY, _oppWidth*10, _oppHeight*10);
		g.setColor(clAqua);
		for (cntr = 0; cntr <= 10; ++cntr) {
			g.drawLine(_oppGridX + _oppWidth*cntr,_oppGridY,
						_oppGridX + _oppWidth*cntr,_oppGridY + (_oppHeight*10));
			g.drawLine(_oppGridX, _oppGridY + _oppHeight*cntr,
						_oppGridX + _oppWidth*10,_oppGridY + _oppWidth*cntr);
    }

    // Draw the ships
    for (cntr = 0; cntr < m_nShipsPlaced; ++cntr)
      if (m_gridOpponent.getShipD(cntr) == 0) // vertical
        g.drawImage(iShipV[cntr],
            _oppGridX+_oppWidth*m_gridOpponent.getShipX(cntr),
            _oppGridY+_oppWidth*m_gridOpponent.getShipY(cntr),
            _oppWidth, Grid.cm_nShipSize[cntr] * _oppHeight, this);
      else
        g.drawImage(iShipH[cntr],
            _oppGridX+_oppWidth*m_gridOpponent.getShipX(cntr),
            _oppGridY+_oppHeight*m_gridOpponent.getShipY(cntr),
            Grid.cm_nShipSize[cntr] * _oppWidth, _oppHeight, this);

    // show any shots
    g.setColor(Color.white);
    for (cntr = 0; cntr < 10; ++cntr)
      for (cntr2 = 0; cntr2 < 10; ++cntr2)
        if (m_gridOpponent.getGrid(cntr, cntr2) > 0) {
             g.fillOval(_oppGridX+cntr*_oppWidth+6,_oppGridY+cntr2*_oppHeight+6,4,4);
             if (m_gridOpponent.getGrid(cntr, cntr2) == 1)
               g.drawImage(iSplash,_oppGridX+cntr*_oppWidth,_oppGridY+cntr2*_oppHeight,_oppWidth,_oppHeight,this);
             else
               g.drawImage(iHit,_oppGridX+cntr*_oppWidth,_oppGridY+cntr2*_oppHeight,_oppWidth,_oppHeight,this);
        }

	}

    
// *** GAME MODE HANDLING ***
	// Handles setting animation state when mode changes
	synchronized private void changeMode(int nMode)
    {
		m_bAnimationPlaying = false;
		m_nMode = nMode;
		switch (nMode) {
			case MODE_TITLE :
				m_bFullDrawNeeded = true;
				break;
			case MODE_PLACESHIPS :
				m_bFullDrawNeeded = true;
				break;
			case MODE_ANIMATEDTRANSFER :
				m_nAnimPhase = 0;
				m_nAnimX = _playGridX;
				m_nAnimY = _playGridY;
				m_nAnimW = _playWidth * 10;
				m_nAnimH = _playHeight * 10;
				m_bFullDrawNeeded = true;
				m_bAnimationPlaying = true;
				break;
			case MODE_PLAYGAME :
				break;
			case MODE_GAMEOVER :
				break;
			case MODE_TITLETRANSFER :
                Dimension d = getSize();
                startGame();
				m_nAnimX = 0;
				m_nAnimY = 0;
				m_nAnimW = d.width;
				m_nAnimH = d.height;
				m_bAnimationPlaying = true;
				break;
			default :
				/* do nothing */
		}
		repaint();
	}

	// Automatically goes to next game mode
	private void nextMode() {
		if (m_nMode == MODE_TITLETRANSFER)
			changeMode(MODE_TITLE);
		else
			changeMode(m_nMode + 1);
	}

    // *************************************************
    // ***** METHODS - Configuration File Handling *****
    // *************************************************

    void loadConfig()
    {
        String s = getParameter("CONFIG");
        if (s == null)
        {
            errorMessage = "Missing properties";
            errorOccured = true;
            return;
        }
        URL url;
        try
        {
            url = new URL(getCodeBase(), s);
        }
        catch (MalformedURLException mue)
        {
            errorMessage = "Invalid URL specified for config";
            errorOccured = true;
            return;
        }

        String temp;
        String command, param;
        int cntr, index;
        BufferedReader in;

        try
        {
            in = new BufferedReader(new InputStreamReader(url.openStream()));
            temp = in.readLine();
            do
            {
                //System.out.println(temp);
                index = temp.indexOf(' ');
                if (index > 2) // must have 3 characters before space
                {
                    command = temp.substring(0, index);
                    param = temp.substring(index+1);
                    //System.out.println("CMD=" + command + " param=" + param);
                    _config.put(command, param);
                }
                temp = in.readLine();
            }
            while (temp != null);
            in.close();
        }
        catch (IOException ioe)
        {
            errorMessage = "IO Exception reading config";
            errorOccured = true;
            return;
        }

        URL host = getCodeBase();
        String sHost = host.getHost().toUpperCase();
        int tempN;

        prepareLayout();
        prepareSounds();
        preloadGraphics();
    }

    private void prepareLayout()
    {
        _playGridX = parseInt((String)_config.get(PLAYGRIDX), _playGridX);
        _playGridY = parseInt((String)_config.get(PLAYGRIDY), _playGridY);
        _playWidth = parseInt((String)_config.get(PLAYWIDTH), _playWidth);
        _playHeight = parseInt((String)_config.get(PLAYHEIGHT), _playHeight);
        _oppGridX = parseInt((String)_config.get(OPPGRIDX), _oppGridX);
        _oppGridY = parseInt((String)_config.get(OPPGRIDY), _playGridY);
        _oppWidth = parseInt((String)_config.get(OPPWIDTH), _oppWidth);
        _oppHeight = parseInt((String)_config.get(OPPHEIGHT), _oppHeight);
        _messageX = parseInt((String)_config.get(MESSAGEX), _messageX);
        _messageY = parseInt((String)_config.get(MESSAGEY), _messageY);
        int fontSize = parseInt((String)_config.get(MESSAGESIZE), 16);
        _messageFont = new Font("Serif", Font.PLAIN, fontSize);
        _shipsX = parseInt((String)_config.get(SHIPSX), _shipsX);
        _shipsY = parseInt((String)_config.get(SHIPSY), _shipsY);
        _shipsSize = parseInt((String)_config.get(SHIPSSIZE), _shipsSize);
        _shipsFont = new Font("Serif", Font.PLAIN, _shipsSize);
    }

    private void prepareSounds()
    {
        _loadedPercent = 4;
        repaint();
        _playSoundToggle = true;
        _soundImage = getImage(getDocumentBase(), parseString(SOUND_TOGGLE));
        preloadImage(_soundImage, 7);
        int nx = parseInt((String)_config.get(SOUND_TOGGLEX), 0);
        int ny = parseInt((String)_config.get(SOUND_TOGGLEY), 0);
        int nsize = parseInt((String)_config.get(SOUND_TOGGLESIZE), 32);
        _soundToggle = new Rectangle(nx, ny, nsize, nsize);

        _soundSilence = getAudioClip(getCodeBase(), parseString(SOUND_SILENCE));
        _soundSplash = getAudioClip(getCodeBase(), parseString(SOUND_SPLASH));
        _soundHit = getAudioClip(getCodeBase(), parseString(SOUND_HIT));
        _soundSink = getAudioClip(getCodeBase(), parseString(SOUND_SINK));
        _soundWin = getAudioClip(getCodeBase(), parseString(SOUND_WIN));
        _soundLose = getAudioClip(getCodeBase(), parseString(SOUND_LOSE));
    }
    
    private void preloadImage(Image i, int pct)
    {
        if (_tracker == null)
            _tracker = new MediaTracker(this);
        _tracker.addImage(i, pct);
        try
        {
            _tracker.waitForID(pct);
        }
        catch (InterruptedException ie)
        {
            System.err.println("Error loading image at " + pct);
        }
        _loadedPercent = pct;
        repaint();
    }
    
    private void preloadGraphics()
    {
        String sName;
        _loadedPercent = 10;
        repaint();
        
        // Load the graphics
        sName = (String)_config.get(TITLE);
        iTitle = getImage(getDocumentBase(), sName);
        preloadImage(iTitle, 20);
        for (int cntr = 0; cntr < 5; ++cntr)
        {
            sName = (String)_config.get(VSHIP + "[" + cntr + "]");
            iShipV[cntr] = getImage(getDocumentBase(), sName);
            preloadImage(iShipV[cntr], cntr*10+25);
            sName = (String)_config.get(HSHIP + "[" + cntr + "]");
            iShipH[cntr] = getImage(getDocumentBase(), sName);
            preloadImage(iShipH[cntr], cntr*10+30);
        }

        sName = (String)_config.get(SPLASH);
        iSplash = getImage(getDocumentBase(), sName);
        preloadImage(iSplash, 75);
        sName = (String)_config.get(HIT);
        iHit = getImage(getDocumentBase(), sName);
        preloadImage(iHit, 80);
        sName = (String)_config.get(LOSE);
        iComputerWins = getImage(getDocumentBase(), sName);
        preloadImage(iComputerWins, 90);
        sName = (String)_config.get(WIN);
        iPlayerWins = getImage(getDocumentBase(), sName);
        preloadImage(iPlayerWins, 100);
    }

    public String[] parseString(String s, char ch)
    {
        int cntr, count = 0;
        int len = s.length();
        char sc;

        for (cntr = 0; cntr < len; ++cntr)
        {
            sc = s.charAt(cntr);
            if (sc == ch)
                ++count;
        }

        String rs[] = new String[count+1];
        int start = 0;
        int last = 0;
        for (cntr = 0; cntr < count; ++cntr)
        {
            last = s.indexOf(ch, start);
            rs[cntr] = s.substring(start, last);
            start = last + 1;
        }
        rs[count] = s.substring(start);

        return rs;
    }

    public int parseInt(String s, int d)
    {
        int r;

        try
        {
            r = Integer.parseInt(s);
        }
        catch (Exception e)
        {
            r = d;
        }

        return r;
    }

    private String parseString(String s)
    {
        String ret = (String)_config.get(s);
        if (ret == null)
            return "";
        else
            return ret;
    }
    // ****************************************
    // ***** METHODS - From MouseListener *****
    // ****************************************

    /** Interface obligation - ignored */
    public void mouseClicked(MouseEvent e)
    {
    }

    public void mousePressed(MouseEvent e)
    {
    }

    public void mouseReleased(MouseEvent e)
    {
        int x = e.getX();
        int y = e.getY();
        int nTemp;

        if (_soundToggle != null)
        {
            if (_soundToggle.contains(x, y))
            {
                _playSoundToggle = !_playSoundToggle;
                repaint();
                return;
            }
        }
        if (m_nMode == MODE_PLACESHIPS) {
            if ( m_gridOpponent.placeShip(m_nShipsPlaced, m_nCurX, m_nCurY, m_nCurD) == true ) {
                ++m_nShipsPlaced;
                m_nOldX = -1;
                m_nOldY = -1;
                if (m_nShipsPlaced == 5) {
                    m_sMessage = "Place your shot...";
                    nextMode();
                }
                repaint();
            }
        } else if (m_nMode == MODE_PLAYGAME) {
            m_sMessage = "Place your shot...";
            if (m_gridPlayer.isValidShot((mouseX-_playGridX) / _playWidth, (mouseY-_playGridY) / _playHeight)) {
                nTemp = m_gridPlayer.shoot((mouseX-_playGridX) / _playHeight, (mouseY-_playGridY) / _playHeight);
                if (nTemp > 1)
                {
                    playSound(_soundSink);
                    m_sMessage = "You sunk my " + Grid.c_szShipNames[nTemp - 2];
                    if (m_gridPlayer.fleetBeenSunk())
                    {
                        playSound(_soundWin);
                        m_nWinState = PLAYERWINS;
                        nextMode();
                        repaint();
                        return;
                    }
                }
                else if (nTemp == 1)
                {
                    playSound(_soundHit);
                }
                else
                {
                    playSound(_soundSplash);
                }
                nTemp = m_gridOpponent.takeShot();
                if (nTemp > 1) {
                    m_sMessage = "I sunk your " + Grid.c_szShipNames[nTemp - 2];
                    if (m_gridOpponent.fleetBeenSunk()) {
                        playSound(_soundLose);
                        m_nWinState = COMPUTERWINS;
                        nextMode();
                    }
                }
                repaint();
            }

        } else {
            nextMode();
        }
        repaint();
    }

    /** Interface obligation - ignored */
    public void mouseEntered(MouseEvent e)
    {
    }

    /** Interface obligation - ignored */
    public void mouseExited(MouseEvent e)
    {
    }

    
    // **********************************************
    // ***** METHODS - From MouseMotionListener *****
    // **********************************************

    /** Interface obligation - ignored */
    public void mouseDragged(MouseEvent e)
    {
    }

    public void mouseMoved(MouseEvent e)
    {
        mouseX = e.getX();
        mouseY = e.getY();
        if (m_nMode == MODE_PLACESHIPS)
            repaint();
    }

    
    // **************************************
    // ***** METHODS - From KeyListener *****
    // **************************************
    /** Interface obligation - ignored */
    public void keyTyped(KeyEvent e)
    {
    }

    /** Interface obligation - ignored */
    public void keyPressed(KeyEvent e)
    {
    }

    public void keyReleased(KeyEvent e)
    {
//        int key = e.getKeyCode();

        if (m_nMode == MODE_PLACESHIPS)
        {
            if (m_nCurD == 0)
                m_nCurD = 1;
            else
                m_nCurD = 0;
        }
        else if (_loadedPercent >= 100)
        {
            nextMode();
        }
        repaint();
    }

    
    // ***********************************
    // ***** METHODS - Sound Support *****
    // ***********************************

    void playSound(AudioClip ac)
    {
        if ( (_playSoundToggle == false) || (ac == null) )
            return;
        ac.play();
    }
    
}

