package com.blazinggames.fleet;

import java.util.*;

/**
 * Fleet
 * Copyright (C) 1998, 2002, 2006 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * The Fleet game grid 
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class Grid {
// *** CONSTANTS ***
	static final int flt_SHIPVERTICAL = 0;
	static final int flt_SHIPHORIZONTAL = 1;

	 static final int cm_nShipSize[] = { 5, 4, 4, 3, 2 };
	 static final int c_nDOffsetX[] = { 1, -1, 0, 0 };
	 static final int c_nDOffsetY[] = { 0, 0, -1, 1 };
	 static final String c_szShipNames[] =
			 {"Battleship", "Carrier", "Destroyer", "Cruser", "Patrol Boat" };

// *** VARIABLES ***
	int m_nShipX[] = new int[5];
	int m_nShipY[] = new int[5];
	int m_nShipD[] = new int[5];
	int m_nShipSunk[] = new int[5];
	int m_nGrid[][] = new int[10][10];

// *** CONSTRUCTORS ***
	public Grid() {
		initGrid();
	}

public void initGrid(){
	   int cntr, cntr2;

	   for (cntr = 0; cntr < 5; ++cntr) {
		   m_nShipX[cntr] = -1;
		   m_nShipY[cntr] = -1;
		   m_nShipD[cntr] = 0;
		   m_nShipSunk[cntr] = 0;
	   }
	   for (cntr = 0; cntr < 10; ++cntr)
		   for (cntr2 = 0; cntr2 < 10; ++cntr2)
			   m_nGrid[cntr][cntr2] = 0;
}

// *** INFORMATION METHODS ***
	public int getShipX(int nID) {
			return m_nShipX[nID];
	}

	public int getShipY(int nID) {
		return m_nShipY[nID];
	}

	public int getShipD(int nID) {
		return m_nShipD[nID];
	}

	public int getSunkState(int nID) {
		return m_nShipSunk[nID];
	}

	public int getGrid(int X, int Y) {
		return m_nGrid[X][Y];
	}

	public boolean isShipInRegion(int nShipID, int x1, int y1, int x2, int y2) {
		int nShipX2, nShipY2;

		// generate coordinates to make ship into box
		if (m_nShipD[nShipID] == 1) {	// Horizontal
			nShipX2 = m_nShipX[nShipID] + cm_nShipSize[nShipID];
			nShipY2 = m_nShipY[nShipID];
		} else { 						// vertical
			nShipY2 = m_nShipY[nShipID] + cm_nShipSize[nShipID];
			nShipX2 = m_nShipX[nShipID];
		}

		// Simple bounding box check
		if ( (nShipX2>=x1) && (m_nShipX[nShipID] <= x2) &&
				(nShipX2>=x1) && (m_nShipX[nShipID] <= x2) )
			return true;
		else
			return false;
	}
	//---------------------------------------------------------------------------
	public boolean checkPlacement(int nID, int X, int Y, int D)
	{
		int nTemp;
		int cntr;
		boolean nValue = true; // Assume valid placement until shown otherwise

		if (D == 1) { // 1 = Horizontal
			nTemp = X + cm_nShipSize[nID] - 1;
			if ( (X + cm_nShipSize[nID]) > 10)
				nValue = false;
			for (cntr = 0; cntr < nID; ++cntr) {
				if (m_nShipD[cntr]!= 0) {
					if ( (m_nShipY[cntr] == Y) &&
								( ( (X >= m_nShipX[cntr]) &&
									(X < (m_nShipX[cntr]+cm_nShipSize[cntr])))
								|| ( (nTemp >= m_nShipX[cntr]) &&
									(nTemp <(m_nShipX[cntr]+cm_nShipSize[cntr]))) ))
						nValue = false;
				} else {
					if (
					(
						(Y >= m_nShipY[cntr]) &&
								(Y < (m_nShipY[cntr]+cm_nShipSize[cntr]))
					) &&
								(
								 (X <= m_nShipX[cntr]) &&
								 (nTemp >= (m_nShipX[cntr]))
								) )
						nValue = false;
				}
			}
		} else { // 0 = Vertical
			nTemp = Y + cm_nShipSize[nID] - 1;
			if ( (Y + cm_nShipSize[nID]) > 10)
				nValue = false;
			for (cntr = 0; cntr < nID; ++cntr) {
				if (m_nShipD[cntr] == 0) {
					if ( (m_nShipX[cntr] == X) &&
								( ( (Y >= m_nShipY[cntr]) &&
									(Y < (m_nShipY[cntr]+cm_nShipSize[cntr]))) ||
								( (nTemp >= m_nShipY[cntr]) &&
									(nTemp <(m_nShipY[cntr]+cm_nShipSize[cntr]))) ))
						nValue = false;
				} else {
					if ( ( (X >= m_nShipX[cntr]) &&
								(X < (m_nShipX[cntr]+cm_nShipSize[cntr])) ) &&
								( (Y <= m_nShipY[cntr]) &&
								(nTemp >= (m_nShipY[cntr])) ) )
						nValue = false;
				}
			}
		}
		return nValue;
	}

// *** MODIFICATION METHODS ***
	//---------------------------------------------------------------------------
	public boolean placeShip(int nID, int X, int Y, int D)
	{
		if ( checkPlacement(nID, X, Y, D) ) {
			m_nShipX[nID] = X;
			m_nShipY[nID] = Y;
			m_nShipD[nID] = D;
			return true;
		} else
			return false;
	}
	//---------------------------------------------------------------------------
	public void randomShips() {
		int cntr, nX, nY, nD;

		Random rand = new Random();
		for (cntr = 0; cntr < 5; ++cntr) {
			do {
				nX = Math.abs(rand.nextInt() % 10);
				nY = Math.abs(rand.nextInt() % 10);
				nD = Math.abs(rand.nextInt() % 2);
			} while (!placeShip(cntr, nX, nY, nD));
		}
	}
	//---------------------------------------------------------------------------
	boolean isValidShot(int X, int Y)
	{
		if ( (X < 0) || (X > 9) || (Y < 0) || (Y > 9) )
			return false;
		if (m_nGrid[X][Y] != 0)
			return false;
		else
			return true;
	}
	//---------------------------------------------------------------------------
	int shoot(int X, int Y)
	{
		int cntr, cntr2, nResult = 0;

		m_nGrid[X][Y] = 1;
		// See if shot has hit ship
		for (cntr = 0; cntr < 5; ++cntr) {
			if (m_nShipD[cntr] != 0) { // Horizontal
				if ( (Y == m_nShipY[cntr]) &&
						( (X >= m_nShipX[cntr]) &&
						(X < (m_nShipX[cntr] + cm_nShipSize[cntr])) ) ) {
					m_nGrid[X][Y] = 2;
					nResult = 2 + cntr;    // assume hit ship sunk
					for (cntr2 = 0; cntr2 < cm_nShipSize[cntr]; ++cntr2)
						if (getGrid(m_nShipX[cntr] + cntr2, Y) == 0)
							nResult = 1;
				}
			} else { // Vertical
				if ( (X == m_nShipX[cntr]) &&
						( (Y >= m_nShipY[cntr]) &&
						(Y < (m_nShipY[cntr] + cm_nShipSize[cntr])) ) ) {
				   m_nGrid[X][Y] = 2;
					nResult = 2 + cntr;    // assume hit ship sunk
					for (cntr2 = 0; cntr2 < cm_nShipSize[cntr]; ++cntr2)
						if (getGrid(X, m_nShipY[cntr] + cntr2) == 0)
							nResult = 1;
				}
			}
		}
		if (nResult > 1)
			m_nShipSunk[nResult - 2] = 1;
		return nResult;
	}
	//---------------------------------------------------------------------------
	boolean fleetBeenSunk()
	{
		int cntr, nSunkCount = 0;

		for (cntr = 0; cntr < 5; ++cntr)
			if (getSunkState(cntr) != 0)
				++nSunkCount;
		if (nSunkCount == 5)
			return true;
		else
			return false;
	}

}
