package com.blazinggames.fleet;

import java.util.*;

/**
 * Fleet
 * Copyright (C) 1998, 2002, 2006 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * The Fleet game opponent ai 
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
class Opponent
extends Grid
{
	public static final int flt_HITLISTSIZE = 18;

// *** VARIABLES ***
	 public int LastShotX, LastShotY;

	int m_nHitListX[];
	int m_nHitListY[];
	int m_nHitListState[];
	int m_nHitListPos, m_nHitsOutstanding, m_nLastShotStatus;
	int m_nShotDirection;

	public Opponent() {
		initGrid();
	}

	public void initGrid() {
		int cntr;

		super.initGrid();
	  m_nHitListX = new int[18];
	  m_nHitListY = new int[18];
    m_nHitListState = new int[18];

		LastShotX = LastShotY = -1;
		for (cntr = 0; cntr < flt_HITLISTSIZE; ++cntr) {
			m_nHitListX[cntr] = 0;
			m_nHitListY[cntr] = 0;
			m_nHitListState[cntr] = 0;
		}
		m_nHitListPos = m_nHitsOutstanding = m_nLastShotStatus = 0;
		m_nShotDirection = 0;
	}
	//---------------------------------------------------------------------------
	int takeShot()
	{
		if (m_nHitsOutstanding != 0)
			targetShip();
		else
			targetShot();
		m_nLastShotStatus = shoot(LastShotX, LastShotY);
		if (m_nLastShotStatus != 0)
			handleHit();
		return m_nLastShotStatus;
	}
	//---------------------------------------------------------------------------
	void targetShip()
	{
		int nNewX = 0, nNewY = 0, cntr, nTarget = 0;
		boolean nCheck;

		if (m_nLastShotStatus == 1) { // Still hitting ship (and not sunk one)
			nNewX = LastShotX + c_nDOffsetX[m_nShotDirection];
			nNewY = LastShotY + c_nDOffsetY[m_nShotDirection];
		}
//		if ( (m_nLastShotStatus != 1) || (isValidShot(nNewX, nNewY)) ){
		if ( (m_nLastShotStatus != 1) || (!isValidShot(nNewX, nNewY)) ){
			for (cntr = 0; cntr < m_nHitListPos; ++cntr)
				if (m_nHitListState[cntr]!= 0) {
					nTarget = cntr;
					break;
				}
			m_nShotDirection = 0;
			do {
				nNewX = m_nHitListX[nTarget] + c_nDOffsetX[m_nShotDirection];
				nNewY = m_nHitListY[nTarget] + c_nDOffsetY[m_nShotDirection];
				nCheck = isValidShot(nNewX, nNewY);
				if (nCheck == false)
					++m_nShotDirection;
			} while (nCheck == false);
		}
		LastShotX = nNewX;
		LastShotY = nNewY;

	}
	//---------------------------------------------------------------------------
	void targetShot()
	{
		Random rand = new Random();
		do {
			LastShotX = Math.abs(rand.nextInt()%10);
			LastShotY = Math.abs(rand.nextInt()%10);
		} while (!isValidShot(LastShotX, LastShotY));
	}
	//---------------------------------------------------------------------------
	void handleHit()
	{
		int cntr, cntr2;

		if (m_nLastShotStatus > 1) { // If sunk ship
			for (cntr = 0; cntr < cm_nShipSize[m_nLastShotStatus - 2]; ++ cntr) {
			   if (m_nShipD[m_nLastShotStatus - 2] != 0) { // Horizontal
					for (cntr2 = 0; cntr2 < m_nHitListPos; ++cntr2)
						if ( (m_nHitListX[cntr2] ==
									(m_nShipX[m_nLastShotStatus - 2] + cntr)) &&
									(m_nHitListY[cntr2] ==
									m_nShipY[m_nLastShotStatus - 2]) )
							m_nHitListState[cntr2] = 0;
				} else { // Vertical
					for (cntr2 = 0; cntr2 < m_nHitListPos; ++cntr2)
						if ( (m_nHitListY[cntr2] ==
									(m_nShipY[m_nLastShotStatus - 2] + cntr)) &&
									(m_nHitListX[cntr2] ==
									m_nShipX[m_nLastShotStatus - 2]) )
							m_nHitListState[cntr2] = 0;
				}
			}
			m_nHitsOutstanding -= (cm_nShipSize[m_nLastShotStatus - 2] - 1);
		} else {
			++m_nHitsOutstanding;
			m_nHitListX[m_nHitListPos] = LastShotX;
			m_nHitListY[m_nHitListPos] = LastShotY;
			m_nHitListState[m_nHitListPos] = 1;
			++m_nHitListPos;
		}
	}
//---------------------------------------------------------------------------
}
