/*
 * Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.wilmag;

import java.applet.Applet;
import java.applet.AudioClip;
import java.io.InputStream;
import java.io.IOException;
import java.awt.Toolkit;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Panel;

//import sun.applet.AppletAudioClip;

/**
 * Wilmag's Lab
 * Interface for handling animations.  Just has the tick function which is
 * called by a controlling class to advance to the next frame.
 * Also includes Constants and global funtions/variables used by Wilmag's Lab.
 *
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: January 8, 1999
*/
public class Anim
extends Panel {
	private static final long serialVersionUID = 8776010100L;
  static final boolean INJAR = false;

  public static final int SCREEN_TITLE = 1;
  public static final int SCREEN_SELECT = 2;
  public static final int SCREEN_INTRODUCTION = 3;
  // Note: The game level id's are directly linked to the # of pieces
  // Which is why they are placed in the following slots
  public static final int SCREEN_GAME_L1 = 4;
  public static final int SCREEN_GAME_L2 = 5;
  public static final int SCREEN_GAME_L3 = 6;
  public static final int SCREEN_GAME_L4 = 7;
  public static final int SCREEN_INTERMISSION1 = 8;
  public static final int SCREEN_INTERMISSION2 = 9;
  public static final int SCREEN_INTERMISSION3 = 10;
  public static final int SCREEN_WIN = 11;
  public static final int SCREEN_LOSE = 12;

  Image loadImage(Applet a, String s) {
    Image i = null;

    if (INJAR) {
      InputStream instreamImage = a.getClass().getResourceAsStream(s);
      if (instreamImage == null) {
        System.err.println("Can not get image from JAR!");
        return null;
      }
      Toolkit tk = Toolkit.getDefaultToolkit();
      try {
        byte imageBytes[] = new byte[instreamImage.available()];
        instreamImage.read(imageBytes);
        instreamImage.close();
        i = tk.createImage(imageBytes);
      } catch (IOException e) {
        System.err.println("Can not get image from JAR!");
      }
    }
    else {
      i = a.getImage(a.getCodeBase(), "res/wilmag/" + s);
    }
      MediaTracker tracker  =  new  MediaTracker(this);
      tracker.addImage(i, 1);
      try  {
        tracker.waitForID(1);
      }  catch  (InterruptedException  e)  {
        System.err.println("Image loading Interrupted Exception");
      }
      if (tracker.isErrorID(1))
        System.err.println("Image loading Error");
//    }

    return i;
  }

  AudioClip loadAudioClip(Applet a, String s) {
    AudioClip au = null;

    au = a.getAudioClip(a.getDocumentBase(), "res/sound/wilmag/" + s);
    return au;
  }

  /**
    Called whenever animation is to advance to the next frame.  Returns status
    information defined in the above global constants.
  */
  public int tick() {
    return 0;
  }
}


