/*
 * Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.wilmag;

import java.applet.*;
import java.awt.*;
//import java.awt.image.*;
//import java.awt.event.*;

// Use my ZONES API
import com.blazinggames.common.zones.*;
//import com.blazinggames.common.sprites.*;

/**
 * Wilmag's Lab
 * Introduction, Intermitions, win and lose animations.
 *
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: January 12, 1999
*/
public class AnimScreen
extends Anim
implements ZoneListener {
// *** CONSTANTS ***
	private static final long serialVersionUID = 8776020100L;
	
  // ANIMATION ID's
  public static final int ANIM_INTRODUCTION = 0;
  public static final int ANIM_INTERMISSION1 = 1;
  public static final int ANIM_INTERMISSION2 = 2;
  public static final int ANIM_INTERMISSION3 = 3;
  public static final int ANIM_WIN = 4;
  public static final int ANIM_LOSE = 5;

  static final int IMAGE_HERO_BEFORE = 0;
  static final int IMAGE_GUARD = 1;
  static final int IMAGE_WILMAG = 2;
  static final int IMAGE_JESTER = 3;
  static final int IMAGE_HERO_AFTER = 4;

  // Difficulty Constants
  static final Font FONT_CAPTION = new Font("TimesRoman", Font.PLAIN, 20);

  static final Color[] COLOR_FRAMES = {
    Color.blue, Color.gray, new Color(255, 0, 255), Color.green, Color.blue
  };

  static final int SLIDE_FRAMES[][] = {
    // INTRODUCTION
    {IMAGE_HERO_BEFORE, IMAGE_WILMAG, IMAGE_HERO_BEFORE, IMAGE_WILMAG},
    // INTERMISSION #1
    {IMAGE_WILMAG, IMAGE_GUARD, IMAGE_WILMAG},
    // INTERMISSION #2
    {IMAGE_WILMAG, IMAGE_GUARD, IMAGE_WILMAG, IMAGE_GUARD, IMAGE_WILMAG},
    // INTERMISSION #3
    {IMAGE_GUARD, IMAGE_WILMAG, IMAGE_GUARD, IMAGE_WILMAG},
    // WIN
    {IMAGE_HERO_AFTER, IMAGE_WILMAG, IMAGE_HERO_AFTER, IMAGE_WILMAG,
    IMAGE_HERO_AFTER, IMAGE_WILMAG},
    // LOSE
    {IMAGE_GUARD, IMAGE_GUARD, IMAGE_JESTER, IMAGE_JESTER}
  };

  static final String SLIDE_TEXT[][][] = {
    // INTRODUCTION
    {
      // Hero before leaving for quest
      {"So you want me to recover the bean", "while you make potions."},
      // Wilmag
      {"That is correct."},
      // Hero before leaving for quest
      {"I wish my Alchemy was better.", "Wish me luck"},
      // Wilmag
      {"Good luck!", "Please hurry back!"}
    },
    // INTERMISSION #1
    {
      // Wilmag
      {"Is my apprentice back?"},
      // Guard
      {"No, sir."},
      // Wilmag
      {"The bean\'s challenges can be tricky...",
      "...I better make a stronger potion"}
    },
    // INTERMISSION #2
    {
      // Wilmag
      {"Is my apprentice back yet?"},
      // Guard
      {"No, sir."},
      // Wilmag
      {"What can be taking him so long?"},
      // Guard
      {"Don\'t know, sir."},
      // Wilmag
      {"I\'m going to have to make a stronger potion."}
    },
    // INTERMISSION #3
    {
      // Guard
      {"He\'s still not back, sir!"},
      // Wilmag
      {"Why do you look so worried?"},
      // Guard
      {"Well, sir, our scouts have detected orcs",
      "marching towards our castle, sir."},
      // Wilmag
      {"My apprentice better get back soon...",
      "...or we are in serious trouble!"}
    },
    // WIN
    {
      // Hero after returning from Quest
      {"Master Wilmag, I\'m back", "Here\'s the bean!"},
      // Wilmag
      {"Good.  Now we can cure Lord Bill", "Where are your guards?"},
      // Hero after returning from Quest
      {"They were killed in the ambush."},
      // Wilmag
      {"Ambush!?"},
      // Hero after returning from quest
      {"It\'s a long story!", "(As told in Coffee Quest 5)"},
      // The end
      {"At least I can save Lord Bill!", "* * * T H E   E N D * * *"}
    },
    // LOSE
    {
      // GUARD
      {"Wilmag would have easily made that potion.",
      "You must be an impostor."},
      // GUARD
      {"I thought so!"},
      // JESTER
      {"And I would have gotten away with it too",
      "If it wasn\'t for those darn ingredients!"},
      // JESTER
      {"* * * T H E   E N D * * *"}
    }
  };

  static final int NEXTSCREEN[] = {
    Anim.SCREEN_GAME_L1, Anim.SCREEN_GAME_L2,
    Anim.SCREEN_GAME_L3, Anim.SCREEN_GAME_L4,
    Anim.SCREEN_TITLE, Anim.SCREEN_TITLE
  };

// *** VARIABLES ***
  Image iWall;
  Image iFrames[] = new Image[5];
  Image iMask;

  Image iAnimBuffer;
  Graphics gAnimBuffer;

	ZoneAdapter za;       // The adapter to use for mouse events

  // Zone (interface) Components
  Zone zoneMore;

  // Screen updating variables
	boolean bFullPaint, bItemsCurrent, bValidPaint;
  boolean bAnimationActive, bGameOver;

  int nFrame, nAnimID, nSlideID;

  int nReturn;

// *** CONSTRUCTORS ***
  // The constructor.  iQuest is an image file featuring the quest items,
  // iButtons is images used for various buttons (needed for the exit button)
	public AnimScreen(Applet a) {
    // Load the images
    iWall = loadImage(a, "wall.jpg");
    iFrames[0] = loadImage(a, "hero_before.gif");
    iFrames[1] = loadImage(a, "guard.gif");
    iFrames[2] = loadImage(a, "wilmag.gif");
    iFrames[3] = loadImage(a, "jester.gif");
    iFrames[4] = loadImage(a, "hero_after.gif");
    iMask = loadImage(a, "hand.gif");

    // Create the user interface elements
    createZones();

		// prepare to handle zones
		za = new ZoneAdapter(this);
    za.add(zoneMore);

    // Use the zone adapter as the mouse listeners
		addMouseListener(za);
		addMouseMotionListener(za);

    // And make sure display will be drawn properly
		bFullPaint = true;
    bItemsCurrent = false;
		requestFocus();
    nAnimID = 0;
    nSlideID = 0;
    bGameOver = false;
    nReturn = 0;
	}

  // Routine to create the user interface elements (zones)
  private void createZones() {
    zoneMore = new Zone(1, 0, 0, 640, 480, Zone.MASKS_CLICKABLE);
  }

  private void paintAnimation(Graphics g) {
    if (iAnimBuffer == null) {
      iAnimBuffer = createImage(640, 400);
      gAnimBuffer = iAnimBuffer.getGraphics();
    }

    nFrame+=5;

//    System.out.println("frame: " + nFrame);


    gAnimBuffer.drawImage(iWall, 0, 0, 640,400, this);
    gAnimBuffer.drawImage(iFrames[3], 200, 400-nFrame, 500, 400, 0,300-nFrame, 300,300, this);
    gAnimBuffer.drawImage(iFrames[2], 200, 100, 500, 400-nFrame, 0, 0, 300, 300-nFrame, this);
    gAnimBuffer.drawImage(iMask, 280, 260-nFrame, this);
    g.drawImage(iAnimBuffer, 0, 0, this);

    if (nFrame >= 250) {
      bAnimationActive = false;
      gAnimBuffer.dispose();
      iAnimBuffer.flush();
      gAnimBuffer = null;
      iAnimBuffer = null;
      paintSlide(g);
    }
  }

  private void paintSlide(Graphics g) {
    // Init
    Color colorOld = g.getColor();
    Font fontOld = g.getFont();

    // first paint the background image and clear the text
    g.drawImage(iWall, 0, 0, 640,400, this);
    g.setColor(Color.black);
    g.fillRect(0, 400, 640, 80);

    // paint the character
    g.drawImage(iFrames[SLIDE_FRAMES[nAnimID][nSlideID]], 200, 100, this);

    // paint the text
    g.setFont(FONT_CAPTION);
    g.setColor(COLOR_FRAMES[SLIDE_FRAMES[nAnimID][nSlideID]]);
    FontMetrics fm = g.getFontMetrics();
    for (int cntr = 0; cntr < SLIDE_TEXT[nAnimID][nSlideID].length; ++cntr)
      g.drawString(SLIDE_TEXT[nAnimID][nSlideID][cntr],
            320 - fm.stringWidth(SLIDE_TEXT[nAnimID][nSlideID][cntr]) / 2,
            425 + 25 * cntr);

    // Clean up
    g.setColor(colorOld);
    g.setFont(fontOld);
  }

  // Displays the contents of the panel.  Most work handled by zone adapter
	public void paint(Graphics g) {
    // Catch invalid repaints (window overlaps, ect bypass the proper update)
    if (bValidPaint)
      bValidPaint = false;
    else
      bFullPaint = true;

    // Make sure that proper orbs being used.
//    if (bItemsCurrent == false)
//      updateInventory();

    if (bFullPaint) { // Total redraw
      bFullPaint = false;
      // Paint the entire screen black for now
      Color oldColor = g.getColor();
      g.setColor(Color.black);
      g.fillRect(0,0, 640, 480);
      g.setColor(oldColor);
      // Now draw the holders
		  za.paintZones(g, this);
    }
    // Redraw any zone that has changed (and the sprite if being dragged)
		za.updateZones(g, Color.black, this);

    if (bAnimationActive)
      paintAnimation(g);
    else
      paintSlide(g);
	}

  // Overridded update to prevent flickering
	public void update(Graphics g) {
    // Tell paint that this is a valid update
    bValidPaint = true;
		paint(g);
	}

  // Handle messages that the Zones Adapter gives up
	public void zoneMessage(int nID, int nMessageCode) {
//    if ( (nID == 99) & (nMessageCode == ZoneListener.ZN_SELECTED))
    if (nMessageCode == ZoneListener.ZN_RELEASED) {
      if (bAnimationActive) {
        nFrame = 250;
        repaint();
        return;
      }
      if ( (nAnimID == ANIM_LOSE) && (nSlideID == 0) ) {
        nFrame = 0;
        bAnimationActive = true;
      }
      ++nSlideID;
      if (nSlideID >= SLIDE_FRAMES[nAnimID].length) {
        nSlideID = SLIDE_FRAMES[nAnimID].length - 1;
        nReturn = NEXTSCREEN[nAnimID];
      }
    }
 		repaint();
	}

  // Routine to handle dragging and dropping of icons (sprites) events
	public void zoneDragDrop(int nIDReceiver, int nIDDropped) {
	}

  public int tick() {
    if (bAnimationActive)
      repaint();
    return nReturn;
  }

  public void setAnim(int nID) {
    nAnimID = nID;
    nSlideID = 0;
    bAnimationActive = false;
    bGameOver = false;
    nReturn = 0;
  }
}

