/*
 * Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.wilmag;

//import java.applet.*;
import java.awt.*;
import java.awt.image.*;
//import java.awt.event.*;
//import java.util.Random;

// Use my ZONES API
//import com.blazinggames.common.zones.*;
import com.blazinggames.common.sprites.*;

/**
 * Wilmag's Lab Move history window
 *
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: January 11, 1999
*/
public class History
implements SpriteProducer {
// *** CONSTANTS ***
  static final int MAX_MIXES = 28;

  static final int SMOKE_COMPONENT = 8;
  static final int FIRE_COMPONENT = 9;

  static final Color HISTORY_YELLOW = new Color(255,255,128);
  static final Color HISTORY_BROWN = new Color(106,53,0);
  static final Font HISTORY_FONT = new Font("TimesRoman", Font.PLAIN, 15);

  static final int HISTORY_WIDTH = 260;//150;
  static final int HISTORY_HEIGHT = 400;//295;

	private static final int NUMBER_X = 2;
	private static final String NUMBER_TXT = "#";
	private static final int MIX_X = 20;
	private static final String MIX_TXT = "Mix";
	private static final int RESULT_X = 140;
	private static final String RESULT_TXT = "Results";
	private static final int ICONSIZE = 16;
	private static final int SOURCEICONSIZE = 16;
	private static final int STARTLINE = 20;
	
// *** VARIABLES ***
  Image iComponents;  // Images for the various components

  // array to hold the previous mixtures
  int aanMix[][] = new int[MAX_MIXES][7];
  int anCorrect[] = new int[7];

  // Arrays to hold the results of the previous mixtures
  int anNumSmoke[] = new int[MAX_MIXES];
  int anNumFlame[] = new int[MAX_MIXES];

  int nCurrentGuess, nMaxGuess;

// *** CONSTRUCTORS ***
	public History(Image i) {
    iComponents = i;
    clear(16);
	}

  public void clear(int nTries) {
    nCurrentGuess = 0;
    nMaxGuess = nTries;
  }

  public void setCorrect(int anMix[]) {
    System.arraycopy(anMix, 0, anCorrect, 0, 7);
  }

  public void addGuess(int anMix[], int nSmoke, int nFlame) {
    System.arraycopy(anMix, 0, aanMix[nCurrentGuess], 0, 7);
//    System.out.println(aanMix[nCurrentGuess][0]);
    anNumSmoke[nCurrentGuess] = nSmoke;
    anNumFlame[nCurrentGuess] = nFlame;
    ++nCurrentGuess;
  }

	// Handles drawing contents at provided coordinates
	public void relativePaint(Graphics g, int nX, int nY, ImageObserver observer){
    int cntr, cntr2, plc;
    String sTemp;

    Color oldColor = g.getColor();
    g.setColor(HISTORY_YELLOW);
    g.fillRect(nX, nY, HISTORY_WIDTH, HISTORY_HEIGHT);
    g.setColor(HISTORY_BROWN);
    Font oldFont = g.getFont();
    g.setFont(HISTORY_FONT);

    g.drawString(NUMBER_TXT, nX+NUMBER_X, STARTLINE-2);
    g.drawString(MIX_TXT, nX+MIX_X, STARTLINE-2);
    g.drawString(RESULT_TXT, nX+RESULT_X, STARTLINE-2);

    for (cntr = 1; cntr <= nMaxGuess; ++cntr) {
      sTemp = cntr + ".";
      g.drawString(sTemp, nX+NUMBER_X, cntr*ICONSIZE+STARTLINE);
    }
/*
    g.drawLine(nX+18, nY, nX+18, nY+HISTORY_HEIGHT);
    g.drawLine(nX+88, nY, nX+88, nY+HISTORY_HEIGHT);
    g.drawLine(nX, nY+14, nX+HISTORY_WIDTH, nY+14);
*/
	g.drawLine(nX+MIX_X-2, nY, nX+MIX_X-2, nY+HISTORY_HEIGHT);
	g.drawLine(nX+RESULT_X-2, nY, nX+RESULT_X-2, nY+HISTORY_HEIGHT);
	g.drawLine(nX, nY+STARTLINE, nX+HISTORY_WIDTH, nY+STARTLINE);

    // For every recorded move
    for (cntr = 0; cntr < nCurrentGuess; ++cntr) {
      // Draw the guess
      for (cntr2 = 0; cntr2 < 7; ++cntr2)
        g.drawImage(iComponents, MIX_X+nX+cntr2*ICONSIZE, STARTLINE+nY+cntr*ICONSIZE, 
					MIX_X+nX+cntr2*ICONSIZE+ICONSIZE, STARTLINE+nY+cntr*ICONSIZE+ICONSIZE,
              		aanMix[cntr][cntr2]*SOURCEICONSIZE-SOURCEICONSIZE, 0,
              		aanMix[cntr][cntr2]*SOURCEICONSIZE, SOURCEICONSIZE, observer);
      plc = nX+RESULT_X;
      // Draw the flames from the guess
      for (cntr2 = 0; cntr2 < anNumFlame[cntr]; ++cntr2) {
        g.drawImage(iComponents, plc, STARTLINE+nY+cntr*ICONSIZE, 
        			plc+ICONSIZE, STARTLINE+ICONSIZE+nY+cntr*ICONSIZE,
              		FIRE_COMPONENT*SOURCEICONSIZE, 0, 
              		FIRE_COMPONENT*SOURCEICONSIZE+SOURCEICONSIZE, SOURCEICONSIZE, observer);
        plc += ICONSIZE;
      }

      // Draw the smoke from the guess
      for (cntr2 = 0; cntr2 < anNumSmoke[cntr]; ++cntr2) {
        g.drawImage(iComponents, plc, STARTLINE+nY+cntr*ICONSIZE, 
        			plc+ICONSIZE, STARTLINE+ICONSIZE+nY+cntr*ICONSIZE,
              		SMOKE_COMPONENT*SOURCEICONSIZE, 0, 
              		SMOKE_COMPONENT*SOURCEICONSIZE+SOURCEICONSIZE, SOURCEICONSIZE, observer);
        plc += ICONSIZE;
      }
    }

    // Now draw the correct combo if lost
    if (nCurrentGuess >= nMaxGuess) {
      g.drawString("cor", nX+2, nY+HISTORY_HEIGHT-2);
      for (cntr2 = 0; cntr2 < 7; ++cntr2)
        g.drawImage(iComponents, MIX_X+nX+cntr2*ICONSIZE, nY+HISTORY_HEIGHT-4-ICONSIZE,
              MIX_X+ICONSIZE+nX+cntr2*ICONSIZE, nY+HISTORY_HEIGHT-4,
              anCorrect[cntr2]*SOURCEICONSIZE-SOURCEICONSIZE, 0,
              anCorrect[cntr2]*SOURCEICONSIZE, SOURCEICONSIZE, observer);
    }

    g.setFont(oldFont);
    g.setColor(oldColor);
  }

	// Returns the width of the component
	public int getWidth() {
    return HISTORY_WIDTH;
  }

	// Returns the height of the component
	public int getHeight() {
    return HISTORY_HEIGHT;
  }

}

