/*
 * Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.wilmag;

import java.applet.*;
import java.awt.*;
//import java.awt.image.*;
//import java.awt.event.*;
import java.util.Random;

// Use my ZONES API
import com.blazinggames.common.zones.*;
import com.blazinggames.common.sprites.*;

/**
 * Wilmag's Lab
 * The actual game screen for the game.
 *
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: January 12, 1999
*/
public class LabScreen
extends Anim
implements ZoneListener {
// *** CONSTANTS ***
	private static final long serialVersionUID = 8776030100L;
	
// *** VARIABLES ***
  Image m_iComponents;  // Images for the various components

  AudioClip auClick, auSmoke, auFire, auWin;

	ZoneAdapter za;       // The adapter to use for mouse events

  // Zone (interface) Components
	ItemHolder zoneItems[] = new ItemHolder[7];
  ItemHolder zoneMix[] = new ItemHolder[7];
  ImageSprite isItems[] = new ImageSprite[7];

  Image iSound;
  ImageButton ibSound;

  StaticZone szHistory;
  History history;

  TextButton tbMixButton;

  // Cooking cauldron display
  Image iFire, iPot, iResult;
  ImageSprite isFire, isPot, isResult;
  StaticZone szFire, szPot, szResult;
  int nFireFrame;
  int nLevel;

  // Flame/Smoke/Win animation variables
  int anSequence[] = new int[7];
  int nCurrentSequence, nNumSequences, nSequenceFrame;
  boolean bPlayingSequence;

  // array to hold the mixture
  int anCorrectMix[] = new int[7];

  // Variables to hold results of last mix attempt
  int nNumSmoke, nNumFlame;

  // Number of tries
  int nMaxTries, nTries, nReturn;

  // Screen updating variables
	boolean bFullPaint, bItemsCurrent, bValidPaint, bSound;
  int nTickCount;

// *** CONSTRUCTORS ***
  // The constructor.  iQuest is an image file featuring the quest items,
  // iButtons is images used for various buttons (needed for the exit button)
	public LabScreen(Applet a) {
    int cntr;

    m_iComponents = loadImage(a, "components.gif");
    iFire = loadImage(a, "fire.jpg");
    iPot = loadImage(a, "pot.jpg");
    iResult = loadImage(a, "results.jpg");
    iSound = loadImage(a, "sound.jpg");
    history = new History(loadImage(a, "smcomp.gif"));

    auClick = loadAudioClip(a, "click.au");
    auFire = loadAudioClip(a, "fire.au");
    auSmoke = loadAudioClip(a, "smoke.au");
    auWin = loadAudioClip(a, "win.au");

    // Create the user interface elements
    createZones();

		// prepare to handle zones
		za = new ZoneAdapter(this);
    for (cntr = 0; cntr < 7; ++cntr) {
      za.add(zoneItems[cntr]);
      za.add(zoneMix[cntr]);
    }
    za.add(szFire);
    za.add(szPot);
    za.add(szResult);
    za.add(szHistory);
    za.add(tbMixButton);
    za.add(ibSound);

    // Use the zone adapter as the mouse listeners
		addMouseListener(za);
		addMouseMotionListener(za);

    startLevel(4,4);

    // And make sure display will be drawn properly
		bFullPaint = true;
    bItemsCurrent = false;
    bPlayingSequence = false;
    bSound = true;
		requestFocus();
	}

  // Routine to create the user interface elements (zones)
  private void createZones() {
    int cntr;
		// create the zones

    // Equipt slots (0-5 holds the player's orbs, 6 holds the doors orb)
    for (cntr = 0; cntr < 7; ++cntr) {
      zoneItems[cntr] = new ItemHolder(cntr+1, 10, 40*cntr+150, 40, 40, 255);
      zoneMix[cntr] = new ItemHolder(cntr+8, 50+40*cntr, 430 ,40, 40, 255);
    }

    // Icons for the components
    for (cntr = 0; cntr < 7; ++cntr) {
      isItems[cntr] = new ImageSprite(m_iComponents, 32*cntr,0,32,32);
      zoneItems[cntr].setItem(isItems[cntr], cntr+1);
    }

    isFire = new ImageSprite(iFire, 0,0,250,80);
    isPot = new ImageSprite(iPot, 0,0,250,140);
    isResult = new ImageSprite(iResult, 0,00, 250, 80);

    szFire = new StaticZone(20, 100, 320, isFire);
    szPot = new StaticZone(21, 100, 150, isPot);
    szResult = new StaticZone(22, 100, 70, isResult);

    ibSound = new ImageButton(97, 10, 10, 32, 32, iSound);

    tbMixButton = new TextButton(99, 390, 430, 260, 40, "*  M  I  X  *");
    szHistory = new StaticZone(98, 390, 0, history);
  }


  // Generate the mix
  public void startLevel(int numComponents, int difficulty) {
    int cntr;
    Random r = new Random();

    nReturn = 0;
    nMaxTries = 3 * difficulty;
    history.clear(nMaxTries);
    nTries = 0;
    nLevel = difficulty;

    for (cntr = 0; cntr < 7; ++cntr) {
      anCorrectMix[cntr] = ((r.nextInt() & Integer.MAX_VALUE) % numComponents) + 1;
      zoneMix[cntr].setItem(null, 0);
      zoneMix[cntr].setVisible(true);
      if (cntr < numComponents)
        zoneItems[cntr].setVisible(true);
      else
        zoneItems[cntr].setVisible(false);
    }

    switch (difficulty) {
      case 4:
        anCorrectMix[1] = 0;
        zoneMix[1].setVisible(false);
      case 5:
        anCorrectMix[6] = 0;
        zoneMix[6].setVisible(false);
      case 6:
        anCorrectMix[0] = 0;
        zoneMix[0].setVisible(false);
      default:
        /* Use all.  Requires nothing! */
    }
    history.setCorrect(anCorrectMix);
    updateMixButton();

    // *** TEST CODE HERE ***
/*
    for (cntr = 0; cntr < 7; ++cntr)
      System.out.print(anCorrectMix[cntr] + " ");
    System.out.println(" ");
*/
  }

  /*
    Checks the mix to see what results are.  The global variables nNumSmoke
    and nNumFlame are set.  Returns true if the player has just won.
    This routine assumes that all the needed components in the passed guess
    have been filled.
  */
  public synchronized boolean checkMix() {
    int work[] = new int[7];
    int result[] = new int[7];
    int cntr, cntr2;

    ++nTries;
    if (nTries >= nMaxTries)
      nReturn = Anim.SCREEN_LOSE;
    System.arraycopy(anCorrectMix, 0, work, 0, 7);

    // First pass - remove any correct values (1)
    for (cntr = 0; cntr < 7; ++cntr)
      if (work[cntr] == zoneMix[cntr].getItemType()) {
        result[cntr] = 1;
        work[cntr] = 0;
      }
      else
        result[cntr] = 0;

    // Second pass - see if smoke (3) or flame (2)
    for (cntr = 0; cntr < 7; ++cntr) {
      if (result[cntr] == 0) {
        result[cntr] = 2;
        for (cntr2 = 0; cntr2 < 7; ++cntr2)
          if (work[cntr2] == zoneMix[cntr].getItemType()) {
            result[cntr] = 3;
            work[cntr2] = 0;
            break;
          } // for cntr2
      } // if result
    } // for cntr

    // now that analysis is done, count the smokes and flames
    nNumFlame = 0;
    nNumSmoke = 0;
    for (cntr = 0; cntr < 7; ++cntr)
      if (result[cntr] == 2)
        ++nNumFlame;
      else if (result[cntr] == 3)
        ++nNumSmoke;

    // now add the component to the history bar
    for (cntr = 0; cntr < 7; ++cntr) {
      work[cntr] = zoneMix[cntr].getItemType();
      if (work[cntr] == 0)
        work[cntr] = 8;
    }
    history.addGuess(work, nNumSmoke, nNumFlame);
    szHistory.invalidate();

    // Set up the animations
    nSequenceFrame = 0;
    nCurrentSequence = 0;
    nNumSequences = 0;
    for (cntr = 0; cntr < nNumFlame; ++cntr)
      anSequence[nNumSequences++] = 0;
    for (cntr = 0; cntr < nNumSmoke; ++cntr)
      anSequence[nNumSequences++] = 1;

    boolean bReturn = false;
    if ( (nNumSmoke == 0) && (nNumFlame == 0) ) {
      anSequence[nNumSequences++] = 2;
      nReturn = nLevel + 4;
      bReturn = true;
    }

    bPlayingSequence = true;

    return bReturn;
  }

  // Displays the contents of the panel.  Most work handled by zone adapter
	public void paint(Graphics g) {

    // Catch invalid repaints (window overlaps, ect bypass the proper update)
    if (bValidPaint)
      bValidPaint = false;
    else
      bFullPaint = true;

    // Make sure that proper orbs being used.
//    if (bItemsCurrent == false)
//      updateInventory();

    if (bFullPaint) { // Total redraw
      bFullPaint = false;
      // Paint the entire screen black for now
      Color oldColor = g.getColor();
      g.setColor(Color.black);
      g.fillRect(0,0, 640, 480);
      g.setColor(oldColor);
      // Now draw the holders
		  za.paintZones(g, this);
//      za.fakeDrag();
    }
    // Redraw any zone that has changed (and the sprite if being dragged)
		za.updateZones(g, Color.black, this);
	}

  // Overridded update to prevent flickering
	public void update(Graphics g) {
    // Tell paint that this is a valid update
    bValidPaint = true;
		paint(g);
	}

  // Handle messages that the Zones Adapter gives up
	public void zoneMessage(int nID, int nMessageCode) {
    if ( (nID == 99) & (nMessageCode == ZoneListener.ZN_RELEASED)) {
      tbMixButton.disable();
      checkMix();
    }
    if ( (nID == 97) & (nMessageCode == ZoneListener.ZN_RELEASED)) {
      bSound = !bSound;
      if (bSound)
        ibSound.setImageOffset(0, 0);
      else
        ibSound.setImageOffset(0,32);
    }
 		repaint();
	}

  // Routine to handle dragging and dropping of icons (sprites) events
	public synchronized void zoneDragDrop(int nIDReceiver, int nIDDropped) {
    // Enable discarding of items
    if (nIDReceiver < 8) {
      if (nIDDropped > 7)
        zoneMix[nIDDropped-8].setItem(null, 0);
    }
    else {
      if (nIDDropped < 8) {
        zoneMix[nIDReceiver-8].setItem(isItems[nIDDropped-1], nIDDropped);
      }
      else {
        zoneMix[nIDReceiver-8].swapHolder(zoneMix[nIDDropped-8]);
      }
    }
    updateMixButton();
    bFullPaint = true;
    if (bSound)
      auClick.play();
	}

  private void updateMixButton() {
    boolean bCanMix = !bPlayingSequence;

    for (int cntr = 0; cntr < 7; ++cntr)
      if (zoneMix[cntr].isVisible())
        if(zoneMix[cntr].getItem() == null)
          bCanMix = false;

    if (bCanMix)
      tbMixButton.enable();
    else
      tbMixButton.disable();
  }


  private void doSequenceAnimation() {
    if ((bSound) && (nSequenceFrame == 0)) {
      if (anSequence[nCurrentSequence] == 0)
        auFire.play();
      else if (anSequence[nCurrentSequence] == 1)
        auSmoke.play();
      else
        auWin.play();
    }
    nSequenceFrame += 5;
    if (nSequenceFrame >= 160) {
      ++nCurrentSequence;
      if (nCurrentSequence >= nNumSequences) {
        bPlayingSequence = false;
        isResult.setClip(0,0,250,80);
        updateMixButton();
      }
      else {
        isResult.setClip(anSequence[nCurrentSequence]*250, 0, 250,80);
      }
      nSequenceFrame = 0;
    }
    else {
      isResult.setClip(anSequence[nCurrentSequence]*250, nSequenceFrame, 250,80);
    }
    szResult.invalidate();
    repaint();
  }

  public int tick() {
    ++nTickCount;
    if ((nTickCount & 1) == 1) {
      nFireFrame = (nFireFrame + 1) % 4;
      isFire.setClip(0, 80*nFireFrame, 250, 80);
      szFire.invalidate();
      za.fakeDrag();
      repaint();
    }
    if (bPlayingSequence) {
      doSequenceAnimation();
      return 0;
    }
    return nReturn;
  }
}


