/*
 * Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.wilmag;

//import java.applet.*;
import java.awt.*;
//import java.awt.image.*;
//import java.awt.event.*;

// Use my ZONES API
import com.blazinggames.common.zones.*;
import com.blazinggames.common.sprites.*;

/**
 * Wilmag's Lab
 * Difficulty selection screen
 *
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: January 12, 1999
*/
public class SelectScreen
extends Anim
implements ZoneListener {
// *** CONSTANTS ***
	private static final long serialVersionUID = 8776040100L;

  // Difficulty Constants
  public static final int DIFFICULTY_NONE = 0;
  public static final int DIFFICULTY_NOVICE = 1;
  public static final int DIFFICULTY_APPRENTICE = 2;
  public static final int DIFFICULTY_MAGE = 3;
  public static final int DIFFICULTY_WIZARD = 4;

  static final Font FONTSELECT = new Font("TimesRoman", Font.BOLD, 40);

  static final String STRING_DIFFICULTY[] = {
    "Novice", "Apprentice", "Mage", "Wizard"
  };

  static final Color COLOR_SELECTNORM[] = {
    new Color(0, 128, 0), new Color(128, 128, 0),
    new Color(128, 0, 0), new Color(128, 0, 128)
  };

  static final Color COLOR_SELECTHIGH[] = {
    new Color(0, 255, 0), new Color(255, 255, 0),
    new Color(255, 0, 0), new Color(255, 0, 255)
  };

  static final Color COLOR_SELECTSEL[] = {
    new Color(128, 255, 128), new Color(255, 255, 128),
    new Color(255, 128, 128), new Color(255, 128, 255)
  };

  static final Color COLOR_SELECTSHADOW[] = {
    new Color(0, 64, 0), new Color(64, 64, 0),
    new Color(64, 0, 0), new Color(64, 0, 64)
  };

// *** VARIABLES ***
	ZoneAdapter za;       // The adapter to use for mouse events

  // Zone (interface) Components
  TextButton tbDifficulty[] = new TextButton[4];
  TextSprite tsCaption;
  StaticZone szCaption;

  // Screen updating variables
	boolean bFullPaint, bItemsCurrent, bValidPaint;

  // Selection variables
  int nSelectedLevel;
  boolean bHasSelected;

// *** CONSTRUCTORS ***
  // The constructor.  iQuest is an image file featuring the quest items,
  // iButtons is images used for various buttons (needed for the exit button)
	public SelectScreen() {
    int cntr;

    // Create the user interface elements
    createZones();

		// prepare to handle zones
		za = new ZoneAdapter(this);
    for (cntr = 0; cntr < 4; ++cntr) {
      za.add(tbDifficulty[cntr]);
    }
    za.add(szCaption);

    // Use the zone adapter as the mouse listeners
		addMouseListener(za);
		addMouseMotionListener(za);

    // And make sure display will be drawn properly
		bFullPaint = true;
    bItemsCurrent = false;
		requestFocus();
    nSelectedLevel = 0;
    bHasSelected = false;
	}

  // Routine to create the user interface elements (zones)
  private void createZones() {
    int cntr;

		// create the zones
    for (cntr = 0; cntr < 4; ++cntr) {
      tbDifficulty[cntr] = new TextButton(
            cntr+1, 0, 150+60*cntr, 640, 60, STRING_DIFFICULTY[cntr]);
      tbDifficulty[cntr].showButtonBox(false);
      tbDifficulty[cntr].setTextColors(COLOR_SELECTNORM[cntr],
            COLOR_SELECTHIGH[cntr], COLOR_SELECTSEL[cntr],
            COLOR_SELECTSHADOW[cntr]);
      tbDifficulty[cntr].setFont(FONTSELECT);
    }

    tsCaption = new TextSprite(FONTSELECT, " ", Color.white,
          640, 100);
    tsCaption.setText("SELECT DIFFICULTY:", TextSprite.ALIGNCENTER);
    tsCaption.setTwoTone(Color.gray, Color.darkGray, new Color(192,192,192), 1);
    tsCaption.setBackground(true, Color.gray);

    szCaption = new StaticZone(5, 0, 0, tsCaption);
  }

  public int getSelected() {
    return nSelectedLevel;
  }

  public void clearSelected() {
    nSelectedLevel = 0;
  }

  // Displays the contents of the panel.  Most work handled by zone adapter
	public void paint(Graphics g) {

    // Catch invalid repaints (window overlaps, ect bypass the proper update)
    if (bValidPaint)
      bValidPaint = false;
    else
      bFullPaint = true;

    // Make sure that proper orbs being used.
//    if (bItemsCurrent == false)
//      updateInventory();

    if (bFullPaint) { // Total redraw
      bFullPaint = false;
      // Paint the entire screen black for now
      Color oldColor = g.getColor();
      g.setColor(Color.black);
      g.fillRect(0,0, 640, 480);
      g.setColor(oldColor);
      // Now draw the holders
		  za.paintZones(g, this);
    }
    // Redraw any zone that has changed (and the sprite if being dragged)
		za.updateZones(g, Color.black, this);
	}

  // Overridded update to prevent flickering
	public void update(Graphics g) {
    // Tell paint that this is a valid update
    bValidPaint = true;
		paint(g);
	}

  // Handle messages that the Zones Adapter gives up
	public void zoneMessage(int nID, int nMessageCode) {
    if (nMessageCode == ZoneListener.ZN_SELECTED) {
      nSelectedLevel = nID;
      bHasSelected = true;
    }
//    if ( (nID == 99) & (nMessageCode == ZoneListener.ZN_SELECTED))
 		repaint();
	}

  // Routine to handle dragging and dropping of icons (sprites) events
	public void zoneDragDrop(int nIDReceiver, int nIDDropped) {
	}

  public int tick() {
    if (bHasSelected) {
      bHasSelected = false;
      return Anim.SCREEN_INTRODUCTION;
//      return Anim.SCREEN_GAME_L1;
    }
    return 0;
  }
}

