/*
 * Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.wilmag;

import java.applet.*;
import java.awt.*;
//import java.awt.image.*;
//import java.awt.event.*;

// Use my ZONES API
import com.blazinggames.common.zones.*;
//import com.blazinggames.common.sprites.*;

/**
 * Wilmag's Lab
 * Title and credits screen
 *
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: January 19, 1999
*/
public class TitleScreen
extends Anim
implements ZoneListener {
// *** CONSTANTS ***
	private static final long serialVersionUID = 8776050100L;
  static final Font FONT_HEADING = new Font("TimesRoman", Font.BOLD, 32);
  static final Font FONT_TEXT = new Font("TimesRoman", Font.PLAIN, 24);

  static final String STRING_CREDITS[][] = {
    {"Programming", "Billy D. Spelchan"},
    {"Graphics", "Mary Caton", "Billy D. Spelchan"},
    {"Animation", "Billy D. Spelchan"},
    {"Story", "Billy D. Spelchan"},
    {"Sound Editing", "Billy D. Spelchan"},
    {"Special Thanks", "Crack Dot Com", "(Golgotha Public Domain Release)"}
  };

  static final int DELAY_CREDIT = 30;
  static final int DELAY_BLANK = 20;

// *** VARIABLES ***
  Image iTitle;
	ZoneAdapter za;       // The adapter to use for mouse events

  // Zone (interface) Components
  Zone zoneClick;
  int nCreditIndex, nLastIndex, nDelay;
  boolean bEndTitle;

  // Screen updating variables
	boolean bFullPaint, bItemsCurrent, bValidPaint;

// *** CONSTRUCTORS ***
  // The constructor.  iQuest is an image file featuring the quest items,
  // iButtons is images used for various buttons (needed for the exit button)
	public TitleScreen(Applet a) {

    // Create the user interface elements
    iTitle = loadImage(a, "title.gif");

    zoneClick = new Zone(1, 0,0, 640,480, Zone.MASKS_CLICKABLE);

		// prepare to handle zones
		za = new ZoneAdapter(this);
    za.add(zoneClick);

    // Use the zone adapter as the mouse listeners
		addMouseListener(za);
		addMouseMotionListener(za);

    // And make sure display will be drawn properly
		bFullPaint = true;
    bItemsCurrent = false;
		requestFocus();
    nCreditIndex = nLastIndex = -1;
    nDelay = 0;
    bEndTitle = false;
    setBackground(Color.black);
	}

  // Displays the contents of the panel.  Most work handled by zone adapter
	public void paint(Graphics g) {
		int cntr;
    Color oldColor = g.getColor();

    // Catch invalid repaints (window overlaps, ect bypass the proper update)
    if (bValidPaint)
      bValidPaint = false;
    else
      bFullPaint = true;

    if (bFullPaint) { // Total redraw
      bFullPaint = false;
      // Paint the entire screen black for now
      g.setColor(Color.black);
      g.fillRect(0,0, 640, 480);
      // Now draw the holders
		  za.paintZones(g, this);
      // and the title text
      g.drawImage(iTitle, 70, 0, this);
      // and force the text to be redrawn
      nLastIndex = -1;
    }
    // Redraw any zone that has changed (and the sprite if being dragged)
		za.updateZones(g, Color.black, this);

    // See if text has changed.  If so, redraw it.
    if (nLastIndex != nCreditIndex) {
      g.setColor(Color.black);
      g.fillRect(0,200,640,280);
      if (nCreditIndex >= 0) {
        Font oldFont = g.getFont();
        g.setFont(FONT_HEADING);
        FontMetrics fm = g.getFontMetrics();
        g.setColor(Color.red);
        g.drawString(STRING_CREDITS[nCreditIndex][0],
              320 - (fm.stringWidth(STRING_CREDITS[nCreditIndex][0])>>1), 350);
        g.setFont(FONT_TEXT);
        fm = g.getFontMetrics();
        g.setColor(Color.yellow);
        for (cntr = 1; cntr < STRING_CREDITS[nCreditIndex].length; ++cntr)
          g.drawString(STRING_CREDITS[nCreditIndex][cntr],
              320 - (fm.stringWidth(STRING_CREDITS[nCreditIndex][cntr])>>1),
              350 + 40*cntr);
        g.setFont(oldFont);
      }
      nLastIndex = nCreditIndex;
    }
    g.setColor(oldColor);
	}

  // Overridded update to prevent flickering
	public void update(Graphics g) {
    // Tell paint that this is a valid update
    bValidPaint = true;
		paint(g);
	}

  // Handle messages that the Zones Adapter gives up
	public void zoneMessage(int nID, int nMessageCode) {
//    if ( (nID == 99) & (nMessageCode == ZoneListener.ZN_SELECTED))
    if (nMessageCode == ZoneListener.ZN_SELECTED)
      bEndTitle = true;
 		repaint();
	}

  // Routine to handle dragging and dropping of icons (sprites) events
	public void zoneDragDrop(int nIDReceiver, int nIDDropped) {
	}

  public int tick() {
    ++nDelay;
    if (nCreditIndex < 0) {
      if (nDelay >= DELAY_BLANK) {
        nCreditIndex = 0;
        nDelay = 0;
      }
    }
    else {
      if (nDelay >= DELAY_CREDIT) {
        ++nCreditIndex;
        if (nCreditIndex >= STRING_CREDITS.length)
          nCreditIndex = -1;
        nDelay = 0;
      }
    }
    repaint();

    if (bEndTitle) {
      bEndTitle = false;
      return Anim.SCREEN_SELECT;
    }
    return 0;
  }
}
