/*
 * Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.wilmag;

import java.applet.*;
import java.awt.*;
//import java.awt.event.*;
//import java.io.*;
//import java.net.*;

/**
 * Wilmag's Lab
 * The actual applet!
 *
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: January 19, 1999
*/
public class Wilmag
extends Applet
implements Runnable {
// *** CONSTANTS ***
	private static final long serialVersionUID = 8776070100L;

// *** VARIABLES ***
  LabScreen labScreen;
  TitleScreen titleScreen;
  SelectScreen selectScreen;
  AnimScreen animScreen;

  Anim screen;

	int nMode;
	AudioClip auSilence;
	boolean threadAlive = true;
	
  Thread runner;

	public void init() {
    labScreen = new LabScreen(this);
    selectScreen = new SelectScreen();
    animScreen = new AnimScreen(this);
    titleScreen = new TitleScreen(this);
	auSilence = getAudioClip(getDocumentBase(), "res/sound/silence.au");
	auSilence.loop();
    screen = titleScreen;

		setLayout(new CardLayout());
    add("title", titleScreen);
    add("anim", animScreen);
    add("select", selectScreen);
		add("game", labScreen);
		requestFocus();
	}

  public void changeScreen(int nScreenID) {
    System.gc();
    switch ( nScreenID ) {
      case Anim.SCREEN_TITLE :
  			((CardLayout)getLayout()).show(this,"title");
        screen = titleScreen;
        break;
      case Anim.SCREEN_SELECT :
        selectScreen.clearSelected();
  			((CardLayout)getLayout()).show(this,"select");
        screen = selectScreen;
        break;
      case Anim.SCREEN_INTRODUCTION :
        animScreen.setAnim(AnimScreen.ANIM_INTRODUCTION);
  			((CardLayout)getLayout()).show(this,"anim");
        screen = animScreen;
        break;
      case Anim.SCREEN_GAME_L1 :
      case Anim.SCREEN_GAME_L2 :
      case Anim.SCREEN_GAME_L3 :
      case Anim.SCREEN_GAME_L4 :
        // The screen id's for the games level directly correspond to the
        // number of components in the puzzle
        labScreen.startLevel(selectScreen.getSelected()+3, nScreenID);
  			((CardLayout)getLayout()).show(this,"game");
        screen = labScreen;
        break;
      case Anim.SCREEN_INTERMISSION1 :
        animScreen.setAnim(AnimScreen.ANIM_INTERMISSION1);
  			((CardLayout)getLayout()).show(this,"anim");
        screen = animScreen;
        break;
      case Anim.SCREEN_INTERMISSION2 :
        animScreen.setAnim(AnimScreen.ANIM_INTERMISSION2);
  			((CardLayout)getLayout()).show(this,"anim");
        screen = animScreen;
        break;
      case Anim.SCREEN_INTERMISSION3 :
        animScreen.setAnim(AnimScreen.ANIM_INTERMISSION3);
  			((CardLayout)getLayout()).show(this,"anim");
        screen = animScreen;
        break;
      case Anim.SCREEN_WIN :
        animScreen.setAnim(AnimScreen.ANIM_WIN);
  			((CardLayout)getLayout()).show(this,"anim");
        screen = animScreen;
        break;
      case Anim.SCREEN_LOSE :
        animScreen.setAnim(AnimScreen.ANIM_LOSE);
  			((CardLayout)getLayout()).show(this,"anim");
        screen = animScreen;
        break;
      default:
        System.err.println("Unknown screen specified!");
    }
  }

	public void start() {
		if (runner == null) {
			runner = new Thread(this);
			runner.start();
		}
	}

	public void run() {
		threadAlive = true;
    int result;
		while (threadAlive) {
      result = screen.tick();
      if (result > 0)
        changeScreen(result);
      try { Thread.sleep(100); }
      catch (InterruptedException e) { }
		}
	}

	public void stop() {
		if (runner != null) {
			threadAlive = false;
			runner = null;
		}
	}

	public String getAppletInfo() {
    	return "Wilmag\'s Lab v2.0.030315. Copyright 1998,1999, 2003 Spelchan Software";
	}
}


